/*************************************************************
Gnome-napster, A napster client for the gnome desktop.
Copyright (C) 2000 Andrew Parker

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Contact Info:
	Mailing Address:
		Andrew Parker
		211 Big Corral Rd.
		Hamilton MT, 59840

	Email Address:
		zaphod42@u.washington.edu
	
***************************************************************************/

/*************************
INCLUDE
*************************/
#include <gnome.h>
#include <glib.h>
#include <pthread.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <errno.h>
#include <unistd.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "lnap.h"
#include "download_thread.h"
#include "login.h"
#include "host.h"
#include "songinfo.h"
#include "util.h"
/*******************************
DATATYPES
********************************/
#define QUEUE_LENGTH  1024

#define MAKE_KEY(x, y, key) { gchar *xnew = g_strdup(x); gchar *ynew = g_strdup(y); g_strdown(xnew); g_strdown(ynew); key = g_strdup_printf("%s-%s", xnew, ynew); g_free(xnew); g_free(ynew);}

char *statuslookup[] = {
	"Localy Queued", "Getting info...", "Remotely Queued", "Ready to download",
	"Connecting...", "Requesting file...", "Downloading...",
	"File complete", "Waiting for connection...", "Uploading...", "Error downloading", NULL
};

typedef struct {
	/* internal data */
	gint tag;		/* gdk_input handler tag */
	unsigned long filebytes;/* the amount recieved so far */


	int filed;		/* the file to put the data in */
	GTimer *t;
/*PUBLIC*/
	/*R/W*/
	EGetStatus status;	/* the current status of the download */
	gchar host[MAX_HOST_LENGTH];
	gchar title[MAX_PATH_LENGTH];	/* the title of the file */
	gchar checksum[MAX_HASH_LENGTH];	/* the md5 checksum */
	unsigned long filesize; /* size of the file */
	gchar *key;
	unsigned int throttle;	/* the value for the throttle */
	gint sock;		/* the socket to recieve from */
	/*RO*/
	double kbps;		/* the speed of the download */

	/* stuff we need to do our job */
	/*R/W*/
	gchar pending_path[MAX_PATH_LENGTH];/* the path to put the data while downloading*/
	gchar final_path[MAX_PATH_LENGTH];	/* the path to put the data when we're done */
	gint shift_value;	/* the amount to shift the path in the title */
} sgetfile;


int def_throttle;
int max_downloads, num_downloads;
int enum_index;

GHashTable *hashes;

char def_pending_path[MAX_PATH_LENGTH];
char def_final_path[MAX_PATH_LENGTH];
/****************************************
PROTOTYPES
****************************************/
static sgetfile *dt_add_getfile(char *host, char *filename);
static sgetfile *dt_getfile_find(char *host, char *filename);
static gboolean dt_getfile_remove(sgetfile *gf);

/* cleanup for a single thread */
static void dt_thread_cleanup(sgetfile *d);

static void dt_downloadproc(sgetfile *d);

static int dt_add_index(sgetfile *gf);
gboolean dt_move_file(char *from, char *to);
static gboolean dt_dequeue(sgetfile *gf);
static gboolean dt_start_download(sgetfile *gf, int sock);
static nap_download *dt_get_info(sgetfile *gf);

/***************************************
FUNCTIONS
***************************************/

/* Startup and Shutdown the threads */
/********************************************
extern gboolean download_threads_init(dt_info *defs)

Start up all of the stuff for the download threads.

RETURNS:
	TRUE is all is ok
	FALSE if something went wrong(memory allocation) 
********************************************/
extern gboolean download_threads_init(dt_info *defs)
{
	FUNCPRINT("Initializing...");
	/* make the hash table */
	hashes = g_hash_table_new(g_str_hash, g_str_equal);
	/* initialize all of the global variables */
	def_throttle = defs->default_throttle;
	max_downloads = QUEUE_LENGTH;
	num_downloads = 0;
	strncpy(def_pending_path, defs->pending_path, MAX_PATH_LENGTH);
	strncpy(def_final_path, defs->final_path, MAX_PATH_LENGTH);
	FUNCPRINT("Default pending is: %s\nDefault final: %s",
			def_pending_path, def_final_path);	
	FUNCPRINT("Done");
	return TRUE;	
}
void dt_remove_wrapper(gpointer key, gpointer value, gpointer data)
{
	sgetfile *gf = value;
	FUNCPRINT("Removing %s, %s", gf->host, gf->title);
	/*dt_getfile_remove(value);*/
	if(gf->tag > 0)
	{
		dt_thread_cleanup(gf);	
	}
	g_free(gf->key);
	g_free(gf);
	FUNCPRINT("Done");
}
/********************************************
extern gboolean download_threads_stop(void)

Clean up after all of the threads, stop the ones that are 
	still running
********************************************/
extern gboolean download_threads_stop(void)
{
	FUNCPRINT("Stopping");
	FUNCPRINT("Hash table is %d entries", g_hash_table_size(hashes));
	/* clean up all of the hash table elements*/
	g_hash_table_freeze(hashes);
	g_hash_table_foreach(hashes, dt_remove_wrapper, NULL);
	g_hash_table_thaw(hashes);
	/* get rid of the hash table */
	g_hash_table_destroy(hashes);
	FUNCPRINT("Done");
	return TRUE;
}
/*******************************************
int download_threads_number_downloads(void)

Return the number of current downloads.
********************************************/
int download_threads_number_downloads(void)
{
	return num_downloads;
}
/* add a download to the queue */
/******************************************
extern int download_threads_enqueue(char *host, char *file)

Add a file to the queue for download

host: the name of the host
file: the name of the file

RETURNS:
	and index number that can be used later to 
	get info about the transfer
	-1 if an error occures
NOTE: 
	The caller will still need to send the download
	request to the server.  This function does not
	do that.
*******************************************/
extern gboolean download_threads_enqueue(char *host, char *filename)
{
	sgetfile *gf=NULL;
	FUNCPRINT("Adding to queue");
	/* check to make sure it doesn't exist first */	
	if((gf=dt_getfile_find(host, filename))!=NULL)
	{
		FUNCPRINT("Already here, returning the current one");
		return TRUE;
	}
	/* add the host and file to the list */
	FUNCPRINT("Adding...");
	if((gf=dt_add_getfile(host, filename))==NULL)
	{
		FUNCPRINT("Error!");
		/* an error has occured */
		return FALSE;
	}	

	/* and just for good measure lets checkup on things */
	/*dt_check_queues();*/
	FUNCPRINT("Done");
	/* return the index of it */
	return TRUE;
}
/* remove a download from the queue */
/*****************************************
extern gboolean download_threads_dequeue(char *host, char *file)

Remove a file from the queue.

host: the name of the host that the file is under
file: the name of the file

RETURNS:
	TRUE if is found it and successfully removed it
	FALSE if it wasn't able to remove it
*****************************************/
extern gboolean download_threads_dequeue(char *host, char *file)
{
	/* search the download_files list for the specified
		host and filename */
	int index;
	sgetfile *gf = NULL;
	if((gf = dt_getfile_find(host, file))==NULL)
	{
		return FALSE;
	}
	/* call download_threads_dequeue_with_index */
	return dt_dequeue(gf);
}
/******************************************
extern gboolean dt_dequeue(sgetfile* gf)

gf: the index of the file to remove

RETURNS:
	See above
******************************************/
static gboolean dt_dequeue(sgetfile *gf)
{
	/* find the entry */
	FUNCPRINT("Dequeueing");
	if(gf->tag > 0)
	{
		FUNCPRINT("Stoping the transfer");
		dt_thread_cleanup(gf);
	}
	else
	{
		FUNCPRINT("Removing");
		/* otherwise just call the download remove function on it */
		if(!dt_getfile_remove(gf)) return FALSE;	
	}	
	FUNCPRINT("Done");
	return TRUE;
}

/* get the info about the download 
	e.g. kbps, filesize, filesize gotten, etc. */
/************************************************
extern nap_download *download_threads_get_info(char *host, char *file)

Get some info about the transfer.

host: the host the file is under
file: the file of the transfer

RETURNS:
	NULL if it wasn't able to find the specified file
	else a pointer to a nap_download struct that contains
		the info about the transfer

NOTE:
	the nap_download struct must be freed with g_free() by the caller
************************************************/
extern nap_download *download_threads_get_info(char *host, char *file)
{
	/* search the download_files list for the specified
		host and filename */
	int index;
	sgetfile *gf = NULL;
/*	FUNCPRINT("Looking for %s, %s", host, file);*/
	if((gf = dt_getfile_find(host, file))==NULL)
	{
		FUNCPRINT("Does not exist");
		return NULL;
	}
	/* call dt_get_info */
	return dt_get_info(gf);
}
/*************************************************
extern nap_download *dt_get_info(sgetfile *gf)


RETURNS:
	See above
NOTE:
	auch
*************************************************/
static nap_download *dt_get_info(sgetfile *gf)
{
	nap_download *np=NULL;
	/* allocate the memory for the nap_download struct */
	if((np = g_malloc0(sizeof(nap_download)))==NULL)
	{
		FUNCPRINT("Error mallocing struct");
		return NULL;
	}
	/* fill in the struct and return it */
	np->status	= gf->status;
	np->kbps	= gf->kbps;
	np->filesize	= gf->filesize;
	np->sizecomplete = gf->filebytes;
	np->sock	= gf->sock;
	np->throttle	= gf->throttle;
	np->shift_value	= gf->shift_value;
	strncpy(np->host, gf->host, MAX_HOST_LENGTH);
	strncpy(np->filename, gf->title, MAX_PATH_LENGTH);
	strncpy(np->md5, gf->checksum, MAX_HASH_LENGTH);
	strncpy(np->pending_path, gf->pending_path, MAX_PATH_LENGTH);
	strncpy(np->final_path, gf->final_path, MAX_PATH_LENGTH);

	return np;
}
/*******************************************************************
extern gboolean download_threads_set_info(const nap_download *info)

Set the info for the download queue node.

info: the nap_download struct that contains the data to set for the node.
	
RETURN:
	TRUE if the data was set successfully
	FALSE if the node wasn't found

NOTE:
	The node is found from the data given in the nap_download struct
		either the index or the host and filename.
*******************************************************************/
extern gboolean download_threads_set_info(nap_download *info)
{
	sgetfile *gf;
	FUNCPRINT("Looking by name");
	if(strlen(info->host)>0 && strlen(info->filename)>0)
	{
	/* find by the name */
		if((gf = dt_getfile_find(info->host, info->filename)) ==NULL)
		{
			FUNCPRINT("Invalid host and filename");
			return FALSE;
		}
	}
	else
	{
		FUNCPRINT("No info to find the node!");
		return FALSE;
	}
	
	if(gf->status == EG_DOWNLOADING)
	{
		FUNCPRINT("File downloading");
		FUNCPRINT("Only changing final_path and throttle");
		gf->throttle = info->throttle;
		strncpy(gf->final_path, info->final_path, MAX_PATH_LENGTH);
		return TRUE;
	}
	else
	{
		FUNCPRINT("Changing info");
		gf->status 	= info->status;
		gf->throttle	= info->throttle;
		gf->sock	= info->sock;
		gf->filesize	= info->filesize;
		gf->shift_value = info->shift_value;
		
		strncpy(gf->checksum, info->md5, MAX_HASH_LENGTH);
		strncpy(gf->pending_path, info->pending_path, MAX_PATH_LENGTH);
		strncpy(gf->final_path, info->final_path, MAX_PATH_LENGTH);
	}
	return TRUE;
}
/* go through the whole queue */
void dt_enum_data(gpointer key, gpointer value, gpointer data)
{
	nap_download *info;
	info = dt_get_info(value);
	((download_enum)data)(info);
}

/***********************************************************
extern nap_download *download_threads_enum_queue(void)

Return info about the next file in the queue.

RETURNS: a pointer to a nap_download struct that contains the info.
	NULL if there are no more

NOTE: the caller must call g_free() to free the memory allocated for the
nap_download struct.
***********************************************************/
extern void download_threads_enum_queue(download_enum func)
{
	g_hash_table_foreach(hashes, dt_enum_data, func);
}
/*********************************************************************
extern gboolean download_threads_start_download(char *host, char *filename, int
sock)

start the download.  The download must first be queued with
download_threads_enqueue().

host: the host to look under
filename: the name of the file to get
sock: the socket to read the data from

RETURNS:
	TRUE if it was able to start the download
	FALSE if it wasn't able to start the download

NOTE: sock must be to the point to read the data for the file
*********************************************************************/
extern gboolean download_threads_start_download(char *host, char *filename, int
sock)
{
	sgetfile *gf = dt_getfile_find(host, filename);
	int index;
	if(gf == NULL)
		return FALSE;

	FUNCPRINT("starting download of %s, %s", host, filename);
	return dt_start_download(gf, sock);
}
/****************************************************************************
extern gboolean dtstart_download(sgetfile *gf, int sock)

Start up the download proc to download the info.  The download struct
have have first been created(enqueued) with download_threads_enqueue().

index: the index to reference the download, given by
	download_threads_enqueue().
sock:  the socket to receive the info from.

RETURNS:
	TRUE: if it successfully created the download thread.
	FALSE: if there was a problem.
****************************************************************************/
static gboolean dt_start_download(sgetfile *gf, int sock)
{
	/* check to make sure it isn't already downloading */
	if(gf->status == EG_DOWNLOADING)
		return FALSE; /* it is already downloading */
	/* set it's socket */
	gf->sock = sock;
	/*start it going */
	dt_downloadproc(gf);
	/* increase the number of downloads */
	num_downloads++; 
	return TRUE;
}
/*************************************************************
static sgetfile *dt_add_getfile(char *host, char *filename)

Add a node to the hosts queue.  

host: the host to add to
filename: the name of the file to add

RETURN:
	pointer the the newly created struct
*************************************************************/
static sgetfile *dt_add_getfile(char *host, char *filename)
{

	sgetfile *gf=NULL;

	if((gf=g_malloc0(sizeof(sgetfile)))==NULL)
	{
		return NULL;
	}
	gf->status = EG_GETINFO;
	strcpy(gf->title, filename);
	strcpy(gf->host, host);
	gf->tag  = -1;	
	strncpy(gf->pending_path, def_pending_path, MAX_PATH_LENGTH);
	strncpy(gf->final_path, def_final_path, MAX_PATH_LENGTH);
	
	/*downloads = g_list_append(downloads, gf);*/
	MAKE_KEY(gf->host, gf->title, gf->key)
	g_hash_table_insert(hashes, gf->key, gf);

	FUNCPRINT("Added Host: %s  Filename: %s", gf->host,
gf->title); 
	return gf;
}
/**************************************************************
static sgetfile *dt_getfile_find(char *host, char *filename)

Find the getfile struct, that matches the host and filename

host: the host to look for
filename: the file to look for

RETURNS:
	the sgetfile struct that was found
	NULL if nothing was found
**************************************************************/
static sgetfile *dt_getfile_find(char *host, char *filename)
{
	gchar *key;
	sgetfile *gf;
	MAKE_KEY(host, filename, key)
	gf = g_hash_table_lookup(hashes, key);
	g_free(key);
	return gf; 	
}
/***********************************************************
static gboolean dt_getfile_remove(sgetfile *gf)

Clean up the sgetfile struct, stop the thread if it has one,
and free the memory

RETURNS:
	TRUE if it was able to remove the struct
	FALSE otherwise
***********************************************************/
static gboolean dt_getfile_remove(sgetfile *gf)
{
	int index;
	if(gf == NULL)
	{
		FUNCPRINT("What the hell is this, a NULL?");
		return FALSE;
	}
	FUNCPRINT("Removing %s, %s", gf->host, gf->title);
	if(gf->tag > 0)
	{
		FUNCPRINT("Stopping thread");
		dt_thread_cleanup(gf);
	}		
	/* remove it from the download_files */
	FUNCPRINT("Removing from table");
	/* remove it from the hash table */
	g_hash_table_remove(hashes, gf->key);	
	/* free the memory */
	g_free(gf->key);
	g_free(gf);
	FUNCPRINT("Done");
	return TRUE;
}
/**********************************************
static void dt_thread_cleanup(void *d)

Cleanup for a single thread.

d: pointer to the sgetfile struct that the thread uses
**********************************************/
static void dt_thread_cleanup(sgetfile *gf)
{
	const int buflen = 2048;
	char buf[buflen];
	char dlpath[MAX_PATH_LENGTH];
	
	FUNCPRINT("Done getting data");
	/* note that you get a "connection reset by peer" at the end
	* of a _successful_transfer_! so it's only an error if the file
	* is incomplete, _and_ there was a recv() error
	*/
	if ((gf->filebytes < gf->filesize)) 
	{
		FUNCPRINT("Error Downloading");
		/*perror("recv");*/
		gf->status = EG_ERROR; 
	}
	else
	{
			/* we made it */
			gf->status = EG_FILECOMPLETE;


		sprintf(dlpath, "%s/%s", gf->pending_path, short_path(gf->title,
							gf->shift_value));
		/*move the file to its new home */
		sprintf(buf, "%s/%s", gf->final_path, short_path(gf->title,
							gf->shift_value));
		dt_move_file(dlpath, buf);
	}
	FUNCPRINT("Cleaning up thread");
	/* decrease the number of downloads */
	num_downloads--;
	/*dec the count in the transfer section*/
	/* FIXME: I don't like doing it this way, it makes
		download threads to dependent on the transfer part */
	download_done();
	/* close the file */
	if(gf->filed)
	{
		close(gf->filed);
		gf->filed = -1;
	}
	/* close the socket */
	if(gf->sock)
	{
		close(gf->sock);
		gf->sock = -1;
	}
	gdk_input_remove(gf->tag);
	gf->tag = -1;
	/* send the message to the server that we are done with a 
		download */
	nap_sendpacket(nap_socknap, NM_DOWNLOAD_COMPLETE, 0, ""); 
	FUNCPRINT("Done");
}
void dt_get_data_cb(gpointer data, gint source, GdkInputCondition condition)
{
	int len;
	sgetfile *gf = data;
	const int buflen = 2048;
	char buf[buflen];
	gdouble elapsed;

	if((len = recv(source, buf, buflen, 0)) <= 0)
	{
		/*TODO: check to see if it was really a failure */
		dt_thread_cleanup(gf);
	}
	/* write, then read, because we have the first bytes from above */
	write(gf->filed, buf, len);
	gf->filebytes += len;
	
	/* TODO: check the validity of this speed calc. */
	elapsed = g_timer_elapsed(gf->t, NULL);
	gf->kbps = (double)(gf->filebytes) / (elapsed * 1024.0);
	if(gf->filebytes >= gf->filesize)
	{ 
		FUNCPRINT("Got whole file");
		dt_thread_cleanup(gf);  /* we got the whole thing */
	}
	/* TODO: add the throttleing code */
}
/*****************************************************
static void *dt_downloadproc(void *d)

The function that is the actuall thread.
(Hopefully the way this is designed it shouldn't be to hard
	to change it to a fork if we ever need to)

gf: the sgetfile struct that the thread should use.

NOTES:
	gf->sock is the socket that the thread needs to
		read from.  It needs to be to the point to 
		grab the file data.
	gf->filesize needs to be set beforehand
	gf->filebytes should be the offset that the download 
		is starting at.  0 to get the whole file, usw.
	
*****************************************************/
static void dt_downloadproc(sgetfile *gf)
{
	int len;
	int sock = gf->sock;
	int flags;
	char dlpath[MAX_PATH_LENGTH];	
	FUNCPRINT("Recieving file: %s", gf->title);

	sprintf(dlpath, "%s/%s", gf->pending_path, short_path(gf->title,
gf->shift_value));
	FUNCPRINT("Putting it here: %s", dlpath); 
	/* make the path to put the file in */
	dt_make_directory(dlpath);
	FUNCPRINT("Opening...");
	/* open up the file */
	if(gf->filebytes == 0)
	{
		if ((gf->filed = open(dlpath, O_WRONLY|O_CREAT, S_IRUSR|S_IWUSR)) < 0) {
			/*perror("open");*/
			gf->status = EG_ERROR;
			return;
		}
	}
	else
	{
		/* this is a resume, open with append */
		if((gf->filed = open(dlpath, O_WRONLY|O_APPEND, 0666)) < 0)
		{
			/*perror("open");*/
			gf->status = EG_ERROR;
			return;
		}
	}

	/* update the status of the file */
	gf->status = EG_DOWNLOADING;
	/* send the message to the server that we are
		starting a download */
	nap_sendpacket(nap_socknap, NM_DOWNLOADING_FILE, 0, "");

	make_blocking(sock);	
	/* make the timer */
	gf->t = g_timer_new();
	/* and start it */
	g_timer_start(gf->t);
	FUNCPRINT("File size is %d", gf->filesize);
	FUNCPRINT("Adding input handler");
	gf->tag = gdk_input_add(gf->sock, GDK_INPUT_READ, dt_get_data_cb, gf);
}
gboolean dt_move_file(char *from, char *to)
{
	FUNCPRINT("Moving %s to %s", from, to);
	/* we'll assume that from is already there */
	if(!dt_make_directory(to)) return FALSE;
	if(rename(from, to)<0) return FALSE;
	FUNCPRINT("Worked!");
	return TRUE;
}
/*************************************************
static gboolean dt_make_directory(char *path)

Make the path.

path: the path to make

RETURNS:
	TRUE if it was successfull
	FALSE otherwise
*************************************************/
gboolean dt_make_directory(char *path)
{
	int i;
	g_assert(path != NULL);
	/* need to start at 1 to avoid problems with making
		the wrong dirs (eg "/" changes to "")*/
	for (i = 1; path[i] != 0; i++) {
		if(path[i] == ':')
		{
			/* trying to make a directory name
				with a colon on vfat fails */
			path[i] = 'c'; /* just switch it for something else */
		}
		if (path[i] == '\\' || path[i] == '/') {
			/* handle subdirs */
			path[i] = 0;
		/*	if(mkdir(path, 0777) < 0)
				return FALSE;*/
			mkdir(path, 0777);
			/*FUNCPRINT("making dir '%s'\n", path);*/
			path[i] = '/';
		}
	}
	return TRUE;
}
/*********************************************
gboolean download_threads_change_status(char *host, char *filename, EGetStatus
status)

Change the status of the specified transfer. 
*********************************************/
gboolean download_threads_change_status(char *host, char *filename, EGetStatus
status)
{	sgetfile *gf;
	FUNCPRINT("Changing status on %s, %s", host, filename);
	if((gf = dt_getfile_find(host, filename)) == NULL)
	{
		FUNCPRINT("Didn't find the download");
		return FALSE;
	}
	gf->status = status;
	return TRUE;
}
