#include <glib.h>
#include <limits.h>
#ifndef __DOWNLOAD_THREAD_H__
#define __DOWNLOAD_THREAD_H__
extern char *statuslookup[];

#define MAX_PATH_LENGTH PATH_MAX
#define MAX_HOST_LENGTH 64
#define MAX_HASH_LENGTH 64

typedef enum {
	EG_QUEUED, EG_GETINFO, EG_REMOTEQUEUEFULL, EG_READY,
	EG_CONNECTING, EG_REQUESTING, EG_DOWNLOADING,
	EG_FILECOMPLETE, EG_AWAITING_CONNECTION, EG_UPLOADING, EG_ERROR
} EGetStatus;

typedef struct 
{
	char host[MAX_HOST_LENGTH];
	char filename[MAX_PATH_LENGTH];
	int index;
	unsigned long sizecomplete;
	double kbps;
	int host_downloads_curr;
/*Changable*/
	EGetStatus status;
	char md5[MAX_HASH_LENGTH];		
	unsigned long filesize;
	
	int sock;
	int throttle;
	
	char pending_path[MAX_PATH_LENGTH];
	char final_path[MAX_PATH_LENGTH];
	int shift_value;
} nap_download;
typedef struct 
{
	int max_downloads;
	int default_throttle;
	
	char pending_path[MAX_PATH_LENGTH];
	char final_path[MAX_PATH_LENGTH];
} dt_info;


typedef void (*download_enum)(nap_download *data);

/* Startup and Shutdown the threads */
extern gboolean download_threads_init(dt_info *defs);
extern gboolean download_threads_stop(void);
int download_threads_number_downloads(void);

/* add a download to the queue */
extern gboolean download_threads_enqueue(char *host, char *file);

/* remove a download from the queue */
extern gboolean download_threads_dequeue(char *host, char *file);

/* get the info about the download 
	e.g. kbps, filesize, filesize gotten, etc. */
extern nap_download *download_threads_get_info(char *host, char *file);
extern gboolean download_threads_set_info(nap_download *info);

/* go through the whole queue */
extern void download_threads_enum_queue(download_enum func);

/* start the download after it is queued */
extern gboolean download_threads_start_download(char *host, char *filename, 
	int sock);


/*************
MACROS
*************/
#define DT_LOCK(x) g_mutex_lock(x);
#define DT_UNLOCK(x) g_mutex_unlock(x);
#define DT_TRYLOCK(x) g_mutex_trylock(x);

#endif
