#include "gtkflist.h"
#include "util.h"
#include <gnome.h>
#include <glib.h>
#include "transfer.h"
/*
#include "download_thread.h"
*/
static char* titles[] = { "File", "Host", "Status" };

enum {
	COL_TITLE, COL_HOST, COL_STATUS, COL_COUNT
};

typedef struct
{
	char host[MAX_HOST_LENGTH];
	char filename[MAX_PATH_LENGTH];
} download_entry;
static void download_ui_cancel_cb(GtkWidget *w, gpointer d);

/* popup text */
static GnomeUIInfo popup_menu[] = {
/*
	GNOMEUIINFO_ITEM_NONE(N_("_Select All"),
		N_("Select all items in the list"),
		selall_cb),

	GNOMEUIINFO_ITEM_NONE(N_("_Song Info"),
		N_("View information about this download"),
		song_info_cb),
*/
	GNOMEUIINFO_ITEM_NONE(N_("_Cancel"),
		N_("Cancel this download"),
		download_ui_cancel_cb),
/*	GNOMEUIINFO_ITEM_NONE(N_("_Host Info"),
		N_("View information about this host"),
		host_info_cb),
	GNOMEUIINFO_END*/
};

static GtkWidget *clist;
/********************************************************************
gboolean download_ui_download_add(char *host, char *title, int shift)

Add a new download to the clist and start the transfer.
*********************************************************************/
gboolean download_ui_download_add(char *host, char *title, int shift)
{
	download_entry *entry = NULL;
	int row;
	gchar *row_text[COL_COUNT] = {"", "", ""};
	nap_download *info;
	
	if((info = transfer_get_info(host, title))!=NULL)
	{
		FUNCPRINT("It is already here");
		g_free(info);
		return TRUE;
	}
	
	entry = g_malloc0(sizeof(download_entry));
	strncpy(entry->host, host, MAX_HOST_LENGTH);
	strncpy(entry->filename, title, MAX_PATH_LENGTH);
	
	if(!transfer_get_file(host, title, shift))
	{
		FUNCPRINT("Error adding file");
		g_free(entry);
		return FALSE;
	}
	row_text[COL_TITLE] = short_path(entry->filename, shift);
	row_text[COL_HOST]  = entry->host;

	row = gtk_clist_append(GTK_CLIST(clist), row_text);
	gtk_clist_set_row_data(GTK_CLIST(clist), row, entry);
	FUNCPRINT("Download added");	
	return TRUE;
}
/***********************************************************
static void download_cancel_cb(GtkWidget *w, gpointer d)

Callback for canceling downloads
***********************************************************/
static void download_ui_cancel_cb(GtkWidget *w, gpointer d)
{
	GList *sel;
	download_entry *entry;

	FUNCPRINT("Canceling Download");
	gtk_clist_freeze(GTK_CLIST(clist));
	/* we'll be using the clist */
 	sel = GTK_CLIST(clist)->selection;
	if(sel == NULL) 
		return;	
	while(sel != NULL)
	{
		entry = gtk_clist_get_row_data(GTK_CLIST(clist), (int)sel->data); 
		g_assert(entry != NULL);


		/* send the cancel request to the thread */
		transfer_cancel(entry->host, entry->filename);
		/* remove the entry from the clist */
		gtk_clist_remove(GTK_CLIST(clist), (int)sel->data); 
		g_free(entry);
		/* go onto the next selected item */
		sel = GTK_CLIST(clist)->selection;	
	}
	gtk_clist_thaw(GTK_CLIST(clist));
	FUNCPRINT("Done");
}
/*************************************************************
static void download_remove_complete_cb(GtkWidget *w, gpointer d)

Callback to remove all entries in the clist that are marked as "File Complete".
*************************************************************/
static void download_ui_remove_complete_cb(GtkWidget *w, gpointer d)
{
	int row = 0;
	download_entry *entry;
	nap_download *info;
	FUNCPRINT("Canceling Finished Downloads");
	/* we'll be using the clist */
	entry=gtk_clist_get_row_data(GTK_CLIST(clist), row); 	
	do{
		if((info = transfer_get_info(entry->host, entry->filename))==NULL)
		{
			gtk_clist_remove(GTK_CLIST(clist), row);
			break;
		}
		if(info->status == EG_FILECOMPLETE)
		{
			FUNCPRINT("Removing completed download");
			transfer_cancel(info->host, info->filename);
			gtk_clist_remove(GTK_CLIST(clist), row);
			g_free(info);
			g_free(entry);
		}
 		else
			row++;

		entry=gtk_clist_get_row_data(GTK_CLIST(clist), row); 
	} while(entry != NULL);

	FUNCPRINT("Done");	
	
}
/***********************************************************
static void download_ui_button_press_cb(GtkWidget *w, GdkEventButton *e, gpointer d)

Show the popup menu
***********************************************************/
static gint download_ui_button_press_cb(GtkWidget *w, GdkEventButton *e, gpointer d) {
	/*FIXME: don't do anything at the moment */
}
/***********************************************************
GtkWidget* downloadtab_create(void)

Make the download tab
***********************************************************/
GtkWidget* downloadtab_create(void) {
	GtkWidget *scroll;
	GtkWidget *vbox, *hbox;
	GtkWidget *b;

	vbox = gtk_vbox_new(FALSE, 10);
/* the buttons on top */
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
		hbox = gtk_hbox_new(FALSE, 10);
		/* Make the button for canceling downloads */
			b = gtk_button_new_with_label("Remove");
		gtk_box_pack_start(GTK_BOX(hbox), b, FALSE, FALSE, 0);
		gtk_signal_connect(GTK_OBJECT(b), "clicked", 
				GTK_SIGNAL_FUNC(download_ui_cancel_cb), NULL);
		/* Make the button to cancel completed downloads */
			b = gtk_button_new_with_label("Remove Complete");
		gtk_box_pack_start(GTK_BOX(hbox), b, FALSE, FALSE, 0);
		gtk_signal_connect(GTK_OBJECT(b), "clicked",
				GTK_SIGNAL_FUNC(download_ui_remove_complete_cb), NULL);
		gtk_widget_set_sensitive(hbox, TRUE);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

/* the clist on bottom */
		scroll = gtk_scrolled_window_new(NULL, NULL);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll), 
				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
			clist = gtk_flist_new_with_titles(sizeof(titles)/sizeof(char*), titles);
			gtk_clist_set_column_width(GTK_CLIST(clist), COL_TITLE, 300);
			gtk_clist_set_column_width(GTK_CLIST(clist), COL_HOST, 50);

			gtk_signal_connect(GTK_OBJECT(clist), "button_press_event",
				GTK_SIGNAL_FUNC(download_ui_button_press_cb), NULL);
		/* make it so that we can select multiple things */
		gtk_clist_set_selection_mode(GTK_CLIST(clist),
						GTK_SELECTION_MULTIPLE);
		gtk_container_add(GTK_CONTAINER(scroll), clist);
	gtk_box_pack_start(GTK_BOX(vbox), scroll, TRUE, TRUE, 0);

	return vbox;
}
/*************************************************
gboolean download_ui_update(gpointer data)

The callback to update the download screen
*************************************************/
gboolean download_ui_update(gpointer data)
{	
	int index;
	char buffer[42];
	download_entry *entry = NULL;
	nap_download *info = NULL;
	gtk_clist_freeze(GTK_CLIST(clist));
	for(index = 0; (entry = gtk_clist_get_row_data(GTK_CLIST(clist),
index)) != NULL; index++)
	{
		if((info = transfer_get_info(entry->host, entry->filename))==NULL)
		{
			gtk_clist_remove(GTK_CLIST(clist), index);
			continue;
		}
			
		gtk_clist_set_text(GTK_CLIST(clist), index, COL_TITLE,
				short_path(info->filename, info->shift_value));	 
		gtk_clist_set_text(GTK_CLIST(clist), index, COL_HOST,
								info->host);
		if(info->status == EG_DOWNLOADING)
		sprintf(buffer, "%.2f/%.2fM @ %.2fKBps", 
			(float)info->sizecomplete/1000000.0,
			(float)info->filesize/1000000.0,
			info->kbps);
		else
			strcpy(buffer, statuslookup[info->status]);
		gtk_clist_set_text(GTK_CLIST(clist), index, COL_STATUS,
						buffer);
		g_free(info);
	}
	gtk_clist_thaw(GTK_CLIST(clist));
	return TRUE;
}
