#include <stdio.h>
#include <unistd.h>
#include <sys/stat.h>
#include "mp3_info.h"
#include "util.h"

int mp3_bitrates_table[16][5] =	{0, 0, 0, 0, 0,
				32, 32, 32, 32, 8,
				64, 48, 40, 48, 16,
				96, 56, 48, 56, 24,
				128, 64, 56, 64, 32,
				160, 80, 64, 80, 64,
				192, 96, 80, 96, 80,
				224, 112, 96, 112, 56,
				256, 128, 112, 128, 64,
				288, 160, 128, 160, 128,
				320, 192, 160, 192, 160,
				352, 224, 192, 224, 112,
				384, 256, 224, 256, 128,
				416, 320, 256, 320, 256,
				448, 384, 320, 384, 320,
				-1, -1, -1, -1, -1};

int mp3_frequencies_table[4][3] ={44100, 22050, 11025,
				48000, 24000, 12000,
				32000, 16000, 8000,
				0, 0, 0};



mp3_info *mp3_get_info(int filed)
{
	#define FRAME_SYNC_MASK 	0xFFE00000
	#define MPEG_VERSION_MASK 	0x00180000
	#define LAYER_DESC_MASK		0x00060000
	#define PROTECTION_BIT_MASK	0x00010000
	#define BITRATE_INDEX_MASK	0x0000F000
	#define SAMPLE_RATE_INDEX_MASK	0x00000C00
	#define PADDING_BIT_MASK	0x00000200
	#define PRIVATE_BIT_MASK	0x00000100
	#define CHANNEL_MODE_MASK	0x000000C0
	#define MODE_EXTENSION_MASK	0x00000030
	#define COPYRIGHT_MASK		0x00000008
	#define ORIGINAL_MASK		0x00000004
	#define EMPHASIS_MASK		0x00000003
	mp3_info *info;
	unsigned char temp;
	int head_number = 0;
	unsigned char bytes[4];
	struct stat stats;
	unsigned int num_frames;
while(1)
{
	while(read(filed, &bytes[0], sizeof(char)) == 1)
	{
		/* check to see if this might be the header */
		if(bytes[0] = 0xFF)
		{
			if(read(filed, &bytes[1], 3) < 3)
			{
				FUNCPRINT("Didn't find the header");
				return NULL;
			}
			else 
			{
				break;
			}
		}
	}
	if((bytes[1] & 0xFB) == 0xFB)
	{
		
		info = g_malloc0(sizeof(mp3_info));
		info->version = 0;
		info->layer = 2;
/* Bitrate */
		temp = (bytes[2] >> 4);
		if(temp == 0 || temp == 0x0F) 
		{
/*			FUNCPRINT("Don't want these values");
*/			g_free(info);
			continue;
		}
/*		FUNCPRINT("Bitrate index is %d", temp);
*/		/* get the bitrate using the chart */
		info->bitrate =
			mp3_bitrates_table[temp][(3*info->version)+info->layer];
/*		FUNCPRINT("Bitrate: %d", info->bitrate);
*/ /*Frequency*/
		temp = (bytes[2]>>2);
		temp &= 0x03; /* only want the last 2 bits */
		info->frequency =
			mp3_frequencies_table[temp][info->version];
		if(info->frequency == -1)
		{
			FUNCPRINT("Bad Frequency");
			g_free(info);
			continue;
		}
/*		FUNCPRINT("Frequency: %d", info->frequency);
*/ /* Padding */
		temp = (bytes[2]>>1);
		temp &= 0x01; /* only want the last bit */
		info->padding = temp;
/*		FUNCPRINT("Padding: %d", info->padding);
*/ /*File Length*/
		fstat(filed, &stats);
		info->file_length = stats.st_size;	
/*		FUNCPRINT("File Length: %d", info->file_length);	
*/ /* Frame Length */
		info->frame_length = ((144000 * info->bitrate)/info->frequency) +
							info->padding;	
/*		FUNCPRINT("Frame Length: %d", info->frame_length);
*/ /* Song Length */
		num_frames = info->file_length/(info->frame_length + 4);
		info->length = (num_frames * 1152 / info->frequency);
/*		FUNCPRINT("Song Length: %d", info->length);
		FUNCPRINT("Done");
*/		return info;	
	}/*end of validity check */
	/* else it isn't a valid header so just keep going */
}  /* end while(1) */	
return NULL;	
}
