/* gnome-napster, a GNOME Napster client.
 * Copyright (C) 1999 Evan Martin <eeyem@u.washington.edu>
 */
#include <stdio.h>
#include <gnome.h>
#include <ctype.h>
#include "lnap.h"
#include "search.h"
#include "transfer.h"
#include "util.h"
#include "ui.h"
#include "gtkflist.h"
#include "gconfig.h"
#include "songinfo.h"
#include "host.h"

#include "../pixmaps/speedgreen.xpm"
#include "../pixmaps/speedyellow.xpm"
#include "../pixmaps/speedred.xpm"

/************************************************
PROTOTYPES
************************************************/
static void popup_add_cb(GtkWidget *w, gpointer d);
static void selall_cb(GtkWidget *w, gpointer d);
static void host_info_cb(GtkWidget *w, gpointer d);
static void getfiles_cb();

/**********************************************
MISC, static globals and such
**********************************************/
#define NO_COMPARE "X"

enum {
        BITRATE_MENU, FREQUENCY_MENU, SPEED_MENU, NUM_MENUS
};

typedef struct
{
        int menu;
        char *text;
} menu_info;

menu_info mp3_bitrates[]= {{BITRATE_MENU,"256"}, 
			{BITRATE_MENU, "192"}, 
			{BITRATE_MENU, "160"}, 
			{BITRATE_MENU, "128"}, 
			{BITRATE_MENU, "112"},
			{BITRATE_MENU, "98"},
			{BITRATE_MENU, "64"},
			{BITRATE_MENU, "56"}, 
			{BITRATE_MENU, "48"}, 
			{BITRATE_MENU, "32"},
			{BITRATE_MENU, "24"}, 
			{BITRATE_MENU, "20"}, 
			{BITRATE_MENU, NULL}};
menu_info mp3_frequencies[] = {{FREQUENCY_MENU, "48000"},
				{FREQUENCY_MENU, "44100"}, 
				{FREQUENCY_MENU, "32000"}, 
				{FREQUENCY_MENU, "24000"},
				{FREQUENCY_MENU, "22050"},
				{FREQUENCY_MENU, "16000"},
				{FREQUENCY_MENU, "16000"},
				{FREQUENCY_MENU, "12000"},
				{FREQUENCY_MENU, "11025"},
				{FREQUENCY_MENU, "8000"},
				{FREQUENCY_MENU, NULL}};
menu_info connection_types[] = {{SPEED_MENU, "Unknown"},
				{SPEED_MENU, "14.4 Modem"},
				{SPEED_MENU, "28.8 Modem"},
				{SPEED_MENU, "33.6 Modem"},
				{SPEED_MENU, "57.6 Modem"},
				{SPEED_MENU, "ISDN 64K"},
				{SPEED_MENU, "ISDN 128K"},
				{SPEED_MENU, "Cable"},
				{SPEED_MENU, "DSL"},
				{SPEED_MENU, "T1"},
				{SPEED_MENU, "T3 or greater"},
				{SPEED_MENU, NULL}};

char *comparatives[] = {"AT LEAST", "AT BEST", "EQUAL TO", NULL};

char *compare_button_text[] = {">=", "<=", "=", NO_COMPARE, NULL};

enum {
        COL_NAME, COL_SIZE, COL_BITRATE, COL_TIME, COL_HOST,
        COL_CONNECTION, COL_COUNT
};
static gchar* clist_titles[] = { "Song Name", "Size", "kbps", "Length",
        "Host", "Connection" };

static GnomeUIInfo popup_menu[] = {
	GNOMEUIINFO_ITEM_NONE(N_("_Get this file"),
			N_("Add this file to the download list"),
			getfiles_cb),
	GNOMEUIINFO_ITEM_NONE(N_("_Add to host list"),
			N_("Add this host to the Host tab"),
			popup_add_cb),
	GNOMEUIINFO_ITEM_NONE(N_("_Select All"),
			N_("Select all items in the list"),
			selall_cb),
	GNOMEUIINFO_ITEM_NONE(N_("_Host Info"),
			N_("View information about this host"),
			host_info_cb),
	GNOMEUIINFO_END
};

typedef struct
{
	GtkWidget *label;
	GtkWidget *compare;
	GtkWidget *select;
} compare_group;

typedef struct
{
        /* the overall container */
        GtkWidget *container_widget;

        /* the bitrate compare widgets */
	compare_group bitrate;
        /* the frequency compare widgets */
	compare_group frequency;
        /* the speed compare widgets */
	compare_group speed;
} search_widgets;

/**********************************************
GLOBALS
Ugh!
**********************************************/
static GtkWidget *entry, *clist;
static GdkPixmap *speedgreen = NULL, *speedyellow = NULL, *speedred = NULL;
static GdkBitmap *speedgreenb = NULL, *speedyellowb = NULL, *speedredb = NULL;
static GList *songlist = NULL;
static int songcount = 0;

search_widgets widgets;
gchar *menu_selections[NUM_MENUS];

/*************************************************
FUCTIONS
*************************************************/

void search_load_pixmaps(void) {
	GtkStyle *style;

	if (speedgreen != NULL) return;
	style = gtk_widget_get_style(app);
	speedgreen = gdk_pixmap_create_from_xpm_d(app->window, 
			&speedgreenb, &style->bg[GTK_STATE_NORMAL], speedgreen_xpm);
	speedyellow = gdk_pixmap_create_from_xpm_d(app->window, 
			&speedyellowb, &style->bg[GTK_STATE_NORMAL], speedyellow_xpm);
	speedred = gdk_pixmap_create_from_xpm_d(app->window, 
			&speedredb, &style->bg[GTK_STATE_NORMAL], speedred_xpm);
}

/* implement column sorting when selecting column headings */
static void 
sort_col(GtkCList *clist, gint column, gint width, gpointer user_data)
{
	switch (column) {
		case COL_SIZE:
			gtk_clist_set_compare_func(clist, songinfo_compare_size);
			break;
		case COL_BITRATE:
			gtk_clist_set_compare_func(clist, songinfo_compare_bitrate);
			break;
		case COL_CONNECTION:
			gtk_clist_set_compare_func(clist, songinfo_compare_connection);
			break;
		case COL_TIME:
			gtk_clist_set_compare_func(clist, songinfo_compare_time);
			break;
		case COL_NAME:
		case COL_HOST:
		default:
			gtk_clist_set_compare_func(clist, NULL); /* default compare func */
	}

	if (column == clist->sort_column) {
		/* switch sort type */
		gtk_clist_set_sort_type(clist, GTK_SORT_DESCENDING + GTK_SORT_ASCENDING
				- clist->sort_type);
	} else {
		/* select column to sort */
		gtk_clist_set_sort_column(clist, column);
	}

	/* go for it */
	gtk_clist_sort(clist);
}

static void list_update(int row) {
	GdkPixmap *pix = NULL; GdkBitmap *bit = NULL;
	char *text;
	ssong_info *i = gtk_clist_get_row_data(GTK_CLIST(clist), row);

	/*FUNCPRINT("row=%d", row);*/
	if (i->conn <= NAP_CONNTYPE_57_6) {
		pix = speedred; bit = speedredb;
	} else if (i->conn <= NAP_CONNTYPE_ISDN_128K) {
		pix = speedyellow; bit = speedyellowb;
	} else if (i->conn <= NAP_CONNTYPE_T3_PLUS) {
		pix = speedgreen; bit = speedgreenb;
	}

	text = short_path(i->title, i->pathshift);
 
	if (pix)
		gtk_clist_set_pixtext(GTK_CLIST(clist), row, COL_NAME, text, 2,
				pix, bit);
	else 
		gtk_clist_set_text(GTK_CLIST(clist), row, COL_NAME, text);
}

static void 
list_add(ssong_info *i) {
	gchar* add[COL_COUNT];
	int row;
	g_assert(i != NULL);
/*FUNCPRINT("i=%p", i);*/
        /* make a pointer to the name of the file */
        add[COL_NAME] = g_strdup_printf("%s", short_path(i->title, i->pathshift));


	 /* then the size of the file */
        add[COL_SIZE] = g_strdup_printf("%lu", i->size);

        /* now the host */
        add[COL_HOST] = g_strdup_printf("%s", i->host);

        /* print out the length of the file */
        add[COL_TIME] = g_strdup_printf("%u:%02u", i->time/60, i->time % 60);

        /* figure out what the connection type is */
        add[COL_CONNECTION] = g_strdup_printf("%s", nap_conntypes[i->conn]);

        /* then make the bitrate into a string */
        add[COL_BITRATE] = g_strdup_printf("%d", i->bitrate);

        /* now append the info to then end of the list */
        row = gtk_clist_append(GTK_CLIST(clist), add);

        /* associate the data in the ssong_info struct with this row
         * of the list */
        gtk_clist_set_row_data(GTK_CLIST(clist), row, i);

        list_update(row);

	/* clean up */
	for(row=0; row < COL_COUNT; row++)
		g_free(add[row]);
	/*FUNCPRINT("Leaving");*/
}

void search_result(char *text) {
	ssong_info *i;
	char buf[50];
	/*FUNCPRINT("%s", text);*/
	/* allocate a new struct to hold the info */
	i = songinfo_new();
	
	sscanf(text, "\"%[^\"]\" %s %lu %d %d %d %s %lu %d",
		i->title, i->checksum, &i->size, &i->bitrate, &i->samprate, &i->time,
		i->host, &i->ip, &i->conn);
	i->pathshift = gnome_config_get_int(CONFIG_PREFIX "/Interface/PathDepth=1");

	songcount++;
	snprintf(buf, 50, "Got %d results...", songcount);
	gtk_clist_set_text(GTK_CLIST(clist), 0, COL_NAME, buf);

	songlist = g_list_append(songlist, i);
}

void search_complete(void) {
	static gchar* add[COL_COUNT];
	GList *l = songlist;

	FUNCPRINT("<no args>");

	memset(add, 0, sizeof(add));
	add[0] = g_strdup_printf("%s", "File not found.");

	gtk_clist_freeze(GTK_CLIST(clist));
	gtk_clist_clear(GTK_CLIST(clist));

	if (songcount == 0) { /* not found */
		gtk_clist_append(GTK_CLIST(clist), add);
	} else {
		/* add the songlist data into the list view */
                do {
                        list_add(l->data);
                } while((l=g_list_next(l)) != NULL);
	}

	gtk_clist_thaw(GTK_CLIST(clist));
	gtk_widget_set_sensitive(widgets.container_widget, TRUE);
	printcn("Search complete.");
	g_free(add[0]);
	/* perhaps ping hosts? */
}

static void getfiles_cb() {
	GList *sel;
	ssong_info *i;

	sel = GTK_CLIST(clist)->selection;
	if (sel == NULL) return;
	while (sel != NULL) {
		i = gtk_clist_get_row_data(GTK_CLIST(clist), (int)sel->data);
		if (i != NULL) /* NULL possible if they select "no matches found" */
			/*download_enqueue(i->host, i->title, i->pathshift);*/
			/*i->index = transfer_get_file(i->host, i->title,
i->pathshift);*/
			if(!download_ui_download_add(i->host, i->title,
i->pathshift))
			{
				/* TODO: pop up and error dialog */
			}	
		sel = sel->next;
	}
}
/*************************************************************
static char *search_get_compare_text(GtkWidget *button)

conver the label text on button from the >=, <=, =, and X to the 
appropriate string to send to the napster server for the search

RETURNS: pointer to the text to send
*************************************************************/
static char *search_get_compare_text(GtkWidget *button)
{
	gchar *button_text[1];
	int index;
	
	/* get the label */
	gtk_label_get(GTK_LABEL(GTK_BUTTON(button)->child), button_text);
	
	/* figure out which one it is */
	for(index=0; compare_button_text[index]!=NULL; index++)
	{
		if(!strcmp(button_text[0], compare_button_text[index])) break;
	}
	/* make sure that we don't have a label that shouldn't exist */
	g_assert(compare_button_text[index]!=NULL);
	/* return the comparative used in the search */
	return comparatives[index];
}
/**************************************************************
static void search_menu_item_activated(GtkWidget *w, gpointer d)

Callback for the option menu menus, saves the text for the menuitem 
that the user selects for easy retrieval later.
**************************************************************/
static void search_menu_item_activated(GtkWidget *w, gpointer d)
{
	menu_info *info=d;
	menu_selections[info->menu]=info->text;
}
/***************************************************************
static void search_change_button_state(GtkWidget *w, gpointer *d)

Callback for the comparative buttons
Changes the text to the next state
***************************************************************/
static void search_change_button_state(GtkWidget *w, gpointer d)
{
	gchar *button_text[1];
	int index;
	/*FUNCPRINT("Changing button state");*/
	gtk_label_get(GTK_LABEL(GTK_BUTTON(w)->child), button_text);
	/*FUNCPRINT("Currently: %s", button_text[0]);*/
	/*find out which one is the current state*/
	for(index=0; compare_button_text[index]!=NULL; index++)
		if(!strcmp(compare_button_text[index], button_text[0])) break;

	g_assert(compare_button_text[index]!=NULL);

	/* move up to the next label */
	index++;
	/* wrap around if we are at the end */
	if(compare_button_text[index]==NULL) 
	{
		/*FUNCPRINT("Wrapping");*/
		index=0;
	}
	if(!strcmp(compare_button_text[index], NO_COMPARE))
		gtk_widget_set_sensitive(GTK_WIDGET(d), FALSE);
	else
		gtk_widget_set_sensitive(GTK_WIDGET(d), TRUE);
	gtk_label_set(GTK_LABEL(GTK_BUTTON(w)->child), compare_button_text[index]);
}
/***************************************************************
static void search_cb(GtkWidget *w, gpointer *d)

Callback to perform the search.  Sends the search string to napster
***************************************************************/
static void search_cb(GtkWidget* w, gpointer *d) {
	char buf[1024];
	char *search = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(entry)->entry));
	int len;
	char *compare_text;
	int speed;
	gchar* add[COL_COUNT];
	int index;

	gnome_entry_save_history(GNOME_ENTRY(entry));
	if (search[0] == 0) return;
	for(index=0; search[index]!='\0'; index++)
		search[index]=tolower(search[index]);	
	sprintf(buf, "Searching for \"%s\"", search);
	memset(add, 0, sizeof(add)); /* initialize all columns to null */
	add[0] = buf;
	
	printcn(buf);
	/* gray it all out */
	gtk_widget_set_sensitive(widgets.container_widget, FALSE);
	/* clear the clist and add the search text */
	gtk_clist_clear(GTK_CLIST(clist));
	gtk_clist_append(GTK_CLIST(clist), add);
	gtk_clist_set_row_data(GTK_CLIST(clist), 0, 0);
/*
Format of the search request:

[FILENAME CONTAINS "artist name"] MAX_RESULTS <max> [FILENAME CONTAINS
"song"] [LINESPEED <compare> <link-type>] [BITRATE <compare> "<br>"] [FREQ
<compare> "<freq>"]
 */
	len = sprintf(buf, "FILENAME CONTAINS \"%s\" MAX_RESULTS 100", search);
	/* do the LINESPEED section */
	if((compare_text=search_get_compare_text(widgets.speed.compare))!=NULL)
        {
		/* convert the string to the number that napster needs */
		for(speed=0; connection_types[speed].text != NULL; speed++)
		{
			if(!strcmp(connection_types[speed].text, menu_selections[SPEED_MENU]))
				break;
		}
		/* add the text to the buf */
		len = sprintf(buf, "%s LINESPEED \"%s\" %d", buf, compare_text, speed);
        }
	/* BITRATE section */
	if((compare_text=search_get_compare_text(widgets.bitrate.compare))!=NULL)
	{
		 len = sprintf(buf, "%s BITRATE \"%s\" \"%s\"", buf, compare_text, 
				menu_selections[BITRATE_MENU]);
	}
	/* FREQ section */
	if((compare_text=search_get_compare_text(widgets.frequency.compare))!=NULL)
        {
                 len = sprintf(buf, "%s FREQ \"%s\" \"%s\"", buf, compare_text,
                                menu_selections[FREQUENCY_MENU]);
        }

	/*printcn(buf);*/
	songlist = g_list_free_all(songlist);
	songcount = 0;

	nap_sendpacket(nap_socknap, NM_SEARCH_REQUEST, len, buf);
}

/* emit the entry's "changed" signal so history gets updated properly 
 * FIXME: this is a hack of sorts. */
static void search_btn_cb(GtkWidget *w, gpointer d) {
	gtk_signal_emit_by_name(GTK_OBJECT(gnome_entry_gtk_entry(GNOME_ENTRY(entry))),
			"activate", entry);
}

static void selall_cb(GtkWidget *w, gpointer d) {
	gtk_clist_select_all(GTK_CLIST(clist));
}

static void host_info_cb(GtkWidget *w, gpointer d) {
	view_host_info(d);
}

static void shift_foreach_cb(gpointer data, gpointer d) {
	int row = (int)data;
	int shift = (int)d;
	int totaldepth = 0;
	int c;
	ssong_info *i = gtk_clist_get_row_data(GTK_CLIST(clist), row);

	if (i == NULL) return;

	if (i->pathshift + shift < 0) i->pathshift = 0;
	else {
		/* figure out how many directories are in
			the path name of the title */
		for (c=0; c<=TITLE_LENGTH; c++) {
			if(i->title[c] == '\0')
				break;
			if((i->title[c] == '\\') | (i->title[c] == '/'))
				totaldepth++;
		}

		/* don't shift too far */
		if(i->pathshift + shift > totaldepth)
			i->pathshift = totaldepth;
		else
			i->pathshift += shift;
	}
	list_update(row);
}

static void shift_cb(GtkWidget *w, gpointer d) {
	if (GTK_CLIST(clist)->selection)
		g_list_foreach(GTK_CLIST(clist)->selection, shift_foreach_cb, d);
}

static gint button_press_cb(GtkWidget *w, GdkEventButton *e, gpointer d) {
	GtkWidget *menu;
	ssong_info *i;
	int row, col;

	if (e->type == GDK_2BUTTON_PRESS) {
		getfiles_cb();
		return TRUE;
	} else if (e->type == GDK_BUTTON_PRESS && e->button == 3) {
		gtk_clist_get_selection_info(GTK_CLIST(clist), e->x, e->y,
				&row, &col);
		i = gtk_clist_get_row_data(GTK_CLIST(clist), row);
		if (i == NULL) return FALSE;
		menu = gnome_popup_menu_new(popup_menu);
		gnome_popup_menu_do_popup_modal(menu, NULL, NULL, e, i->host);
		gtk_widget_destroy(menu);
		return TRUE;
	}

	return FALSE;
}

static void popup_add_cb(GtkWidget *w, gpointer d) {
	host_addhost((char*)d);
}
static void button_addhost_cb(GtkWidget *w, gpointer d) {
	ssong_info *i;
	GList *sel;
	FUNCPRINT("adding");
	if ((sel = GTK_CLIST(clist)->selection) == NULL) return;
	i = gtk_clist_get_row_data(GTK_CLIST(clist), (int)sel->data);
	host_addhost(i->host);
	FUNCPRINT("%s added", i->host);
}
/************************************************************************************
GtkWidget * search_create_compare_widget(char *label, GtkWidget *table,
					int table_row, int menu, menu_info *menu_data)

create an entry in the table for the comparatives for the search tab

label:		the label to use on the row
table:		the table widget to pack into
table_row:	to row to put the widgets in
menu:		the menu to use
menu_data:	pointer to the menu_info array to use
group:		pointer to the compare group struct to store the widgets in

RETURNS:	nothing
************************************************************************************/
static void search_create_compare_widget(char *label, GtkWidget *table, 
					int table_row, int menu, 
					menu_info *menu_data, compare_group *group)
{
	GtkWidget *option_menu_item, *option_menu_menu;
	int index;

	/* make the label */
	group->label = gtk_label_new(label);

	/*set the label justification to the right */
	gtk_misc_set_alignment(GTK_MISC(group->label), 1, 0.5);
	/* put the label in the table */
	gtk_table_attach(GTK_TABLE(table), group->label, 0, 1, table_row,
							table_row+1, GTK_FILL,
		GTK_FILL, 0, 0);
	/* make the menu */
	option_menu_menu=gtk_menu_new();
	menu_selections[menu] = menu_data[0].text;
	for(index=0; menu_data[index].text!=NULL; index++)
	{
		option_menu_item = gtk_menu_item_new_with_label(menu_data[index].text);

		gtk_menu_append(GTK_MENU(option_menu_menu), option_menu_item);
		/* this is not a very nice way to do it, but I can't find any
			other way*/
		gtk_signal_connect(GTK_OBJECT(option_menu_item), "activate",
		GTK_SIGNAL_FUNC(search_menu_item_activated), &menu_data[index]);
	}

	/* make the button */
	group->compare = gtk_button_new_with_label(compare_button_text[3]);
	gtk_button_set_relief(GTK_BUTTON(group->compare), GTK_RELIEF_NONE);
	/* put it in the table */
	gtk_table_attach(GTK_TABLE(table), group->compare, 1, 2, table_row, table_row+1,
		GTK_FILL, GTK_FILL, 0, 0);

	/* make the option menu */
	group->select = gtk_option_menu_new();
	gtk_option_menu_set_menu(GTK_OPTION_MENU(group->select), option_menu_menu);
	/* put it in the table */
	gtk_table_attach(GTK_TABLE(table), group->select, 2, 4, table_row, table_row+1, 
		GTK_FILL, GTK_SHRINK, 0, 0);
	gtk_widget_set_sensitive(group->select, FALSE);

	/* connect the button up to the clicked signal to change the label when
		it is clicked */
	gtk_signal_connect(GTK_OBJECT(GTK_BUTTON(group->compare)),
		"clicked", GTK_SIGNAL_FUNC(search_change_button_state), group->select);
}
GtkWidget* create_search(void) {
	GtkWidget *vbox, *hbox;
	GtkWidget *vbox_controls; 
	/* the 4 labels: Search, Bitrate, Frequency, and Speed */
	GtkWidget *label[4];
	GtkWidget *bsearch, *bget, *bhost;
	GtkWidget *bselall;
	GtkWidget *scroll;
	GtkWidget *bshiftleft, *bshiftright;
	GtkWidget *cmdbox, *cmdbox2, *cmdbox3;

	vbox = gtk_vbox_new(FALSE, 10);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);

	
         /* line 1 of search options */
	 cmdbox = gtk_hbox_new(FALSE, 10);

         /* line 2 of search options */
	cmdbox2 = gtk_table_new(1, 1, TRUE);

/* Make the search label */	 
	label[0] = gtk_label_new("Search:");
	gtk_box_pack_start(GTK_BOX(cmdbox), label[0], FALSE, FALSE, 0);
/* Make the Search entry box */
	entry = gnome_entry_new("search");
	gnome_entry_load_history(GNOME_ENTRY(entry));
	gtk_signal_connect(GTK_OBJECT(gnome_entry_gtk_entry(GNOME_ENTRY(entry))),"activate", 
		GTK_SIGNAL_FUNC(search_cb), NULL);
 	gtk_box_pack_start(GTK_BOX(cmdbox), entry, FALSE, FALSE, 0);


/* Bitrate Prompt */
search_create_compare_widget("Bitrate is", cmdbox2, 0, BITRATE_MENU,
		 mp3_bitrates, &widgets.bitrate);
/* Frequency Prompt */
search_create_compare_widget("Frequency is", cmdbox2, 1, FREQUENCY_MENU, 
		mp3_frequencies, &widgets.frequency);
/* Speed Prompt */
search_create_compare_widget("Speed is", cmdbox2, 2, SPEED_MENU,
		connection_types, &widgets.speed);
/*
AT LEAST
AT BEST
EQUAL TO

  speeds:
0  unknown
                1  14.4 kbps
                2  28.8 kpbs
                3  33.6 kbps
                4  56.7 kbps
                5  64K ISDN
                6  128K ISDN
                7  Cable
                8  DSL
                9  T1
                10 T3 or greater
  bitrate:
    256
    192
     160
     128
     112
98
64
56
48
32
24
20

  frequency
    48000
44100
32000
24000
22050
16000
12000
11025
8000

*/ /* end testing */


	cmdbox3 = gtk_hbox_new(FALSE, 10);
	 bsearch = gtk_button_new_with_label("Search");
	gtk_signal_connect(GTK_OBJECT(bsearch), "clicked", GTK_SIGNAL_FUNC(search_btn_cb), &widgets);
	gtk_box_pack_start(GTK_BOX(cmdbox3), bsearch, FALSE, FALSE, 0);

	 bget = gtk_button_new_with_label("Get File(s)");
	 gtk_signal_connect(GTK_OBJECT(bget), "clicked",GTK_SIGNAL_FUNC(getfiles_cb), NULL);

		gtk_box_pack_start(GTK_BOX(cmdbox3), bget, FALSE, FALSE, 0);
			bselall = gtk_button_new_with_label("Select All");
			gtk_signal_connect(GTK_OBJECT(bselall), "clicked",
					GTK_SIGNAL_FUNC(selall_cb), NULL);
		gtk_box_pack_start(GTK_BOX(cmdbox3), bselall, FALSE, FALSE, 0);
			bhost = gtk_button_new_with_label("Add Host");
			gtk_signal_connect(GTK_OBJECT(bhost), "clicked",
					GTK_SIGNAL_FUNC(button_addhost_cb), NULL);
		gtk_box_pack_start(GTK_BOX(cmdbox3), bhost, FALSE, FALSE, 0);


/* start putting the stuff somewhere */
	widgets.container_widget = gtk_hbox_new(FALSE, 10);
	vbox_controls = gtk_vbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(vbox_controls), cmdbox, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox_controls), cmdbox3, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(widgets.container_widget), vbox_controls, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(widgets.container_widget), cmdbox2, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), widgets.container_widget, FALSE, FALSE, 0);

/* Make the widget to list the search results */
		scroll = gtk_scrolled_window_new(NULL, NULL);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll), 
				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
			/* note that this is an flist, not a vanilla clist */
			clist = gtk_flist_new_with_titles(sizeof(clist_titles)/sizeof(char*), 
					clist_titles);
				hbox = gtk_hbox_new(FALSE, 0);
					bshiftleft = songinfo_arrow_button_new(app, TRUE);
					gtk_signal_connect(GTK_OBJECT(bshiftleft), "clicked",
							GTK_SIGNAL_FUNC(shift_cb), (gpointer)-1);
				gtk_box_pack_start(GTK_BOX(hbox), bshiftleft, FALSE, FALSE, 0);
					bshiftright = songinfo_arrow_button_new(app, FALSE);
					gtk_signal_connect(GTK_OBJECT(bshiftright), "clicked",
							GTK_SIGNAL_FUNC(shift_cb), (gpointer)1);
				gtk_box_pack_start(GTK_BOX(hbox), bshiftright, FALSE, FALSE, 0);
				gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(clist_titles[0]), 
						FALSE, FALSE, 5);
			gtk_widget_show_all(hbox);
			gtk_clist_set_column_widget(GTK_CLIST(clist), 0, hbox);
			gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_MULTIPLE);
			gtk_clist_set_column_justification(GTK_CLIST(clist), COL_SIZE,
					GTK_JUSTIFY_RIGHT);
			gtk_clist_set_column_justification(GTK_CLIST(clist), COL_BITRATE,
					GTK_JUSTIFY_RIGHT);
			gtk_clist_set_column_justification(GTK_CLIST(clist), COL_TIME,
					GTK_JUSTIFY_RIGHT);
			gtk_clist_set_column_width(GTK_CLIST(clist), COL_NAME, 200);
			gtk_clist_set_column_width(GTK_CLIST(clist), COL_SIZE, 50);
			gtk_clist_set_column_width(GTK_CLIST(clist), COL_HOST, 60);
			gtk_clist_set_column_width(GTK_CLIST(clist), COL_CONNECTION, 65);
			/*gtk_clist_set_column_width(GTK_CLIST(clist), COL_BITRATE, 25);*/
			gtk_signal_connect(GTK_OBJECT(clist), "button_press_event",
					GTK_SIGNAL_FUNC(button_press_cb), NULL);
			gtk_signal_connect(GTK_OBJECT(clist), "click-column",
					GTK_SIGNAL_FUNC(sort_col), NULL);
			gtk_clist_set_sort_column(GTK_CLIST(clist), 0);	/* sort on title by default */
			gtk_clist_set_auto_sort(GTK_CLIST(clist), TRUE);	
		gtk_container_add(GTK_CONTAINER(scroll), clist);
	gtk_box_pack_start(GTK_BOX(vbox), scroll, TRUE, TRUE, 0);
	gtk_widget_set_sensitive(widgets.container_widget, FALSE);
	search_load_pixmaps();
	return vbox;
}

void search_login_status(int loggedin) {
	if (loggedin) {
		gtk_widget_set_sensitive(widgets.container_widget, TRUE);
		gtk_clist_clear(GTK_CLIST(clist));
	} else {
		gtk_widget_set_sensitive(widgets.container_widget, FALSE);
		gtk_clist_clear(GTK_CLIST(clist));
	}
}
