/* gnome-napster, a GNOME Napster client.
 * Copyright (C) 2000 Andrew Parker <zaphod42@u.washington.edu>
 */
/*************************************************************
Gnome-napster, A napster client for the gnome desktop.
Copyright (C) 2000 Andrew Parker


This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Contact Info:
	Mailing Address:
		Andrew Parker
		211 Big Corral Rd.
		Hamilton MT, 59840

	Email Address:
		zaphod42@u.washington.edu
	
***************************************************************************/

#include <gnome.h>
#include <glib.h>
#include <sys/types.h>
#include <errno.h>
#include <unistd.h>

/* open */
#include <sys/stat.h>
#include <fcntl.h>
/* network */
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "transfer.h"
#include "login.h"
#include "lnap.h"
/*#include "download.h"*/
#include "download_thread.h"
#include "gconfig.h" /* download path */
#include "util.h"
#include "upload_thread.h"
/****************************************************
PROTOTYPES
****************************************************/
#define INVALID_REQUEST_STR	"INVALID REQUEST"
#define GET_STR			"GET"
typedef struct
{
	int tag; /* the tag for the input handler */
	char buffer[2048]; /* buffer to hold the info as we get it*/
	char host[64];
	char filename[MAX_PATH_LENGTH];
	long filesize;
	long offset;
} transfer_info;

static gint localtag = -1;
static int max_downloads;
static int num_downloads;
/****************************************************
FUNCTIONS
****************************************************/
void transfer_input_accept_cb(gpointer data, gint source, 
			GdkInputCondition condition);

void transfer_input_cb(gpointer data, gint source, GdkInputCondition
condition);
void transfer_alt_cb(gpointer data, gint source, GdkInputCondition condition);
void transfer_upload_cb(gpointer data, gint souce, GdkInputCondition
condition);
void transfer_connection_cb(gpointer data, gint source, GdkInputCondition
							condition);

gboolean transfer_check_queues(gpointer data);
void transfer_send_getinfo_request(char *host, char *filename);

/*********************************************************
void transfer_input_accept_cb(gpointer data, gint source, 
			GdkInputCondition condition)

Accept the incoming connects and create a GdkInputHandeler for
them
*********************************************************/
void transfer_input_accept_cb(gpointer data, gint source, 
			GdkInputCondition condition)
{
	int new_sock;
	struct sockaddr sock_addr;
	int addr_size = sizeof(sock_addr);
	transfer_info *info;
	FUNCPRINT("Accepting connection");
	/* accept the connection */
	if((new_sock = accept(source, &sock_addr, &addr_size)) < 0)
	{
		FUNCPRINT("Error getting connection");
		return;
	}
	/* set the new socket to non-blocking */
/*	if((new_sock = make_nonblocking(new_sock))<0)
*/
	if((new_sock = make_blocking(new_sock))<0)
	{
		FUNCPRINT("Can't make blocking");
		close(new_sock);
		return;
	}
	FUNCPRINT("Sending '1'");
	/* send out the 1 for connection confermation */
	send(new_sock, "1",sizeof(char), 0);
	/* make the new transfer */
	info = g_malloc0(sizeof(transfer_info));
	FUNCPRINT("Setting up new input handler");
	/* setup the new handler */
	info->tag = gdk_input_add(new_sock, GDK_INPUT_READ, transfer_input_cb, info);
	FUNCPRINT("Done");
}
/*****************************************************************************
void transfer_input_cb(gpointer data, gint source, GdkInputCondition condition)

Once a connection is accepted this will handle getting the request
******************************************************************************/
void transfer_input_cb(gpointer data, gint source, GdkInputCondition condition)
{
	int len;
	transfer_info *info = data;
	int offset;
	/* read in the info */
	FUNCPRINT("Getting info from connection");
	if((len = recv(source, info->buffer, 3, MSG_PEEK))<0)
	{
		if(errno == EAGAIN)
		{
			/* this is fine, just nothing to read */
			return;
		}
		else g_assert_not_reached();
	}
	info->buffer[len] = '\0';
	FUNCPRINT("Recv'd %s", info->buffer);
	/*make a decision about if we need to read it in */
	if(!strncmp(info->buffer, "SEN", 3))
	{
		FUNCPRINT("Getting ready for a SEND");
		/* remove this handler */
		gdk_input_remove(info->tag);
		memset(info->buffer, 0, 6);
		/* setup the handler for an alternate download */
		info->tag = gdk_input_add(source, GDK_INPUT_READ,
					transfer_alt_cb, info);	
		/* then actually read it from the socket */
		recv(source, info->buffer, 4, 0);
	}
	else if(!strncmp(info->buffer, "GET", 3))
	{
		FUNCPRINT("Getting ready for a GET");
		/* remove this handler */
		gdk_input_remove(info->tag);
		memset(info->buffer, 0, 6);
		/* setup the handler for an upload */
		info->tag = gdk_input_add(source, GDK_INPUT_READ,
					transfer_upload_cb, info);
		/* then actually read it from the socket */
		recv(source, info->buffer, 3, 0);
	}
	else
	{
		FUNCPRINT("Unkown response");
		/* I don't know what they are sending
			but it isn't correct */
		gdk_input_remove(info->tag);
		close(source);
		g_free(info);
		return;
	}
	FUNCPRINT("Done");
}
/******************************************************
void transfer_alt_cb(gpointer data, gint source, GdkInputCondition condition)

Callback to recieve the data after the SEND command and then start the download
when we know what it is.
*******************************************************/
void transfer_alt_cb(gpointer data, gint source, GdkInputCondition condition)
{
	transfer_info *info = data;
	int len;
	nap_download *download_info = NULL;
	/* <nick> "<filename>" <size> */
	/* read in the data */
	FUNCPRINT("Getting data");
	if((len = recv(source, info->buffer, 2048, 0))<0)
	{
		close(source);
		gdk_input_remove(info->tag);
		g_free(info);
		return;
		/* TODO:do the error check */
	} 
	info->buffer[len] = '\0';
	FUNCPRINT("They sent: %s", info->buffer);
	/*split it up */
	sscanf(info->buffer, "%s \"%[^\"]\" %d", info->host, info->filename, &info->filesize);
	/* it should all come in one packet so this should work fine,
		I would think */
	/* look it up and start it */
	FUNCPRINT("Getting info about %s, %s", info->host, info->filename);
	if((download_info = download_threads_get_info(info->host,
						info->filename))==NULL)
	{
		FUNCPRINT("They had an invalid request");
		/* we don't have that file queued */
		send(source, INVALID_REQUEST_STR, 
				strlen(INVALID_REQUEST_STR), 0);
		/* clean up */
		close(source);
		gdk_input_remove(info->tag);
		g_free(info);
		return;	
	}
	download_info->filesize = info->filesize;
	FUNCPRINT("Starting download of %d bytes", download_info->filesize);
	download_threads_set_info(download_info);
	send(source, "0", sizeof(char), 0);
	download_threads_start_download(info->host, info->filename,
								source);
	gdk_input_remove(info->tag);	
	g_free(info);
	g_free(download_info);
}
/*******************************************************
void transfer_upload_cb(gpointer data, gint souce, GdkInputCondition condition)

Callback to recieve the data after the GET command 
and then start the upload when we know what it is.
********************************************************/
void transfer_upload_cb(gpointer data, gint source, GdkInputCondition condition)
{

	int len; 
	char buffer[2048];
	transfer_info *info = data;
	share_info *u_info = NULL;
	FUNCPRINT("Getting an upload request");
	if((len = recv(source, buffer, 2048, 0)) < 0)
	{
		FUNCPRINT("Error getting request");
		gdk_input_remove(info->tag);
		g_free(info);
		close(source);
		return;
	}
	buffer[len] = '\0';	
	sscanf(buffer, "%s \"%[^\"]\" %d", info->host, info->filename,
							&info->offset);
	FUNCPRINT("Request is \"%s\"", buffer);
	if((u_info = upload_threads_get_share_info(info->filename)) == NULL)
	{
		FUNCPRINT("Request for an unshared file");
		send(source, INVALID_REQUEST_STR, strlen(INVALID_REQUEST_STR),
									0);
		gdk_input_remove(info->tag);
		g_free(info);
		close(source);
		return;
	}
	sprintf(buffer, "%d", u_info->filesize);
	send(source, buffer, strlen(buffer), 0);
	FUNCPRINT("Starting upload to %s of %s starting at %d", info->host,
						info->filename, info->offset);
	if(!upload_threads_start_upload(source, info->host, 
		u_info->actual_filename, info->offset))
	{
		FUNCPRINT("Couldn't start upload");
		close(source);
	}

	g_free(u_info);
	gdk_input_remove(info->tag);
	g_free(info);
}
/******************************************
gboolean transfer_threads_init(int port)

Initialize the data structs, open up the port to listen on 
and add the input handeler for it.
******************************************/
gboolean transfer_threads_init(int port, int download_max, char *pending_path,
char *final_path)
{
	dt_info defs;
	/* create a socket and bind it to the 
		specified port 
		WARNING: this might cause problems 
		with SOCKS5(FIXME)*/
	transfer_reset_local_port(port);

	strcpy(defs.pending_path, pending_path);
	strcpy(defs.final_path, final_path);
	defs.default_throttle = 0;
	max_downloads = download_max;
	num_downloads = 0;

	FUNCPRINT("Max of %d downloads", max_downloads);
	FUNCPRINT("Initializing threads");
	if(!download_threads_init(&defs)) return FALSE;
	if(!upload_threads_init()) return FALSE;

	FUNCPRINT("Done");
	g_timeout_add(1000, transfer_check_queues, NULL);	
	return TRUE;	 
}
/*****************************************
gboolean transfer_threads_destroy(void)

clean up all of the threads.
*****************************************/
gboolean transfer_threads_destroy(void)
{
	/* stop all of the current threads */
	FUNCPRINT("Stoping threads");
	download_threads_stop();
	upload_threads_destroy();
	/* remove the input handler */
	gdk_input_remove(localtag);
	localtag = -1;
	/* close the socket */
	close(nap_socklocal);
	FUNCPRINT("Done");
}
/******************************************
void transfer_connection_cb(gpointer data, gint source, GdkInputCondition
							condition)

When the connection to the remote host is made we try to
get the download going.

Standard input and output of an input handler

need to g_free the data when we are done with it.
******************************************/
void transfer_connection_cb(gpointer data, gint source, GdkInputCondition
							condition)
{
	char buffer[1024];
	transfer_info *info = data;
	nap_download *d_info = NULL;
	int option;
	int index;
	int size = sizeof(int);
	/* check to make the sure the connection is valid */
	FUNCPRINT("Checking if we are connected yet");
	getsockopt(source, SOL_SOCKET, SO_ERROR, &option, &size);
	if(option != 0)
	{
		FUNCPRINT("Nope, not connected yet");
		/* not done connecting yet */
		/* FIXME: do some more checking for error here */
		return;
	}
	if((d_info = download_threads_get_info(info->host,
info->filename))==NULL)
	{
		gdk_input_remove(info->tag);
		close(source);
		g_free(info);
		return;
	}
	make_blocking(source);
	/* get the 1 */
	for(index=0; index<10; index++)
	{
		recv(source, &buffer[0], sizeof(char), 0);
		FUNCPRINT("They sent %c", buffer[0]);
		if(buffer[0] == '1') break;
	}

/*	if(buffer[0] != '1')*/
	if(index == 10)
	{
		FUNCPRINT("Don't understand the response");
		gdk_input_remove(info->tag);
		download_threads_change_status(info->host, info->filename,
							EG_ERROR);
		close(source);
		g_free(info);
		g_free(d_info);
		num_downloads--;
		return;
	}

	/* send that we want to GET a file */
	send(source, GET_STR, strlen(GET_STR), 0);
	
	/* now send who we are and what we want */
	sprintf(buffer, "%s \"%s\" %d", 
			g_logininfo.user, info->filename, info->offset);
	FUNCPRINT("Sending: %s", buffer);
	send(source, buffer, strlen(buffer), 0);
	/* FIXME: should probable split this loop into another input
		handler */
	for(index = 0; ;index++)
	{
		recv(source, &info->buffer[index], sizeof(char), MSG_PEEK);
		if(!isdigit(info->buffer[index]) || info->buffer[index]<=0) 
		{
			if(index==0)
			{
				FUNCPRINT("Error getting file");
				download_threads_change_status(info->host, 
						info->filename, EG_ERROR);
				gdk_input_remove(info->tag);
				close(source);
				g_free(info);
				g_free(d_info);
				num_downloads--;
				return;
			}
			else break;
		} 
		recv(source, &info->buffer[index], sizeof(char), 0);
	}
	/*FIXME: we need to find a way to handle the error conditions */
	info->buffer[index] = '\0';
	if(index<=1)
	{
		FUNCPRINT("Error");
		gdk_input_remove(info->tag);
		/* set the error on the download */
		download_threads_change_status(info->host, info->filename,
							EG_ERROR);
		g_free(info);	
		g_free(d_info);
		close(source);
		num_downloads--;
		return;
	}
	FUNCPRINT("They sent back: %s", info->buffer);
	sscanf(info->buffer, "%d", d_info->filesize);
	download_threads_set_info(d_info);
	/* start the download */
	FUNCPRINT("Starting download");
	download_threads_start_download(info->host, info->filename, source);
	g_free(d_info);
	/* clean up */
	gdk_input_remove(info->tag);
	g_free(info);
	FUNCPRINT("Done");
}
/*****************************************
Callbacks for messages from the napster server
*****************************************/
/******************************************
int transfer_getinfo_cb(snap_packet *p)

extract the info from the string, find the 
download, and set the info.
******************************************/
int transfer_getinfo_cb(snap_packet *p)
{
	char host[64];
	glong ip, port;
	char filename[MAX_PATH_LENGTH];
	char md5[64];
	int speed;
	int sock;
	struct sockaddr_in addr; 
	transfer_info *info;
	/* get the info */
	FUNCPRINT("Got info: %s", p->txt); 
	/*<nick> <ip> <port> "<filename>" <md5> <linespeed> */
	sscanf(p->txt, "%s %d %d \"%[^\"]\" %s %d", 
		host, &ip, &port, filename, md5, &speed);
	FUNCPRINT("Connection to %d:%d", ip, port);
	/*need to check to see if this download is in the
		queue and set it to "Connecting" if it is.  Otherwise
		we need to return -1. */
	if(!download_threads_change_status(host, filename, EG_CONNECTING))
	{
		FUNCPRINT("The download isn't here");
		return -1;
	}
	if(port)
	{
		FUNCPRINT("Starting a normal download");
		/* make a new socket */
		if((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0)
		{
			/* an error occured making the socket */
			FUNCPRINT("Unable to make socket");
			return -1;	
		}
		/* set the socket to non-blocking */
		make_nonblocking(sock);
		/* start making the connection */
		addr.sin_family = AF_INET;
		addr.sin_port = htons(port);
		addr.sin_addr.s_addr = htonl(ip);
		connect(sock, &addr, sizeof(struct sockaddr_in));
		/* make the transfer_info struct */
		info = g_malloc0(sizeof(transfer_info));
		strncpy(info->host, host, 64);
		strncpy(info->filename, filename, MAX_PATH_LENGTH);
		info->offset = 0;
		FUNCPRINT("Input handler for: %s, %s", info->host,
info->filename);
		/* create a gdk_input handler for the socket */
		info->tag = gdk_input_add(sock, GDK_INPUT_WRITE |
GDK_INPUT_EXCEPTION,
			transfer_connection_cb, info);
		/*num_downloads++;*/ 
	}
	/* else if the port of the remote host is 0 */
	else
	{
		FUNCPRINT("Need an alternate download");
		/* send the message to the server asking for an
			alternate download */
		nap_sendpacketf(nap_socknap, NM_ALT_DOWNLOAD_REQUEST, 
						"%s \"%s\"", host, filename);	
	}
	return 0;
}
/*******************************************
gboolean transfer_remote_host_full_cb(snap_packet *p)

Callback for a remote host full
*******************************************/
gboolean transfer_remote_host_full_cb(snap_packet *p)
{
	char host[MAX_HOST_LENGTH];
	char filename[MAX_PATH_LENGTH];
	glong filesize;
	gint number;
	nap_download *info;
	/* <nick> "<filename>" <filesize> <number> */
/*	FUNCPRINT("Remote host full");
	FUNCPRINT("They said: %s", p->txt); */
	sscanf(p->txt, "%s \"%[^\"]\" %d %d", host,
			filename, &filesize, &number);
	/* check to see if it still exists and change the status */	
	if(!download_threads_change_status(host, filename, EG_REMOTEQUEUEFULL))
	{
		FUNCPRINT("The download isn't here");
		return FALSE;
	}
	/* just send the download request again */
/*	FUNCPRINT("Sending request again"); */
	nap_sendpacketf(nap_socknap, NM_FILEINFO_REQUEST, 
			"%s \"%s\"", host, filename);
	/* THINK:should we maybe to a wait on it? */
	return TRUE;
}
/********************************************
gboolean transfer_get_error_cb(snap_packet *p)

Callback for an error on the attempt to download
********************************************/
gboolean transfer_get_error_cb(snap_packet *p)
{
	char host[64];
	char filename[MAX_PATH_LENGTH];
	/* Change the specified file to error */
	/* <nick> "<filename>" */
	FUNCPRINT("Error on: %s", p->txt);
	sscanf(p->txt, "%s, \"%[^\"]\"", host, filename);
	if(download_threads_change_status(host, filename, EG_ERROR))
	{	
		num_downloads--;
		return TRUE;
	}
	return FALSE;
}
/***************************
Download Control 
***************************/
/**************************************************
int transfer_get_file(char *host, char *filename)

Add the file to the download queue and start getting it.
**************************************************/
gboolean transfer_get_file(char *host, char *filename, int shift)
{
	nap_download *info;
	/* add the info to the download queue */
	FUNCPRINT("Adding %s \"%s\" to queue", host, filename);
	if(!download_threads_enqueue(host, filename))
	{
		FUNCPRINT("Failed!");
		return FALSE;
	}
	/*TODO: set the info for the transfer */
	info = transfer_get_info(host, filename);
	info->shift_value = shift;
	FUNCPRINT("Shift set to: %d", shift);
	download_threads_set_info(info);
	/* check to see if we have reached the limit */
	FUNCPRINT("Send request?");
	if(num_downloads < max_downloads)
	{
		transfer_send_getinfo_request(host, filename);	
	}
	else
		/*set it to localy queued and setup a handler
			to check every once in a while and try to start one */
	{
		download_threads_change_status(host, filename, EG_QUEUED);
		FUNCPRINT("No, done");
	}
	g_free(info);
	return TRUE;
}
/*****************************************************
nap_download *transfer_get_info_with_index(int index)

Get info about the transfer
*****************************************************/
nap_download *transfer_get_info_with_index(int index)
{
	FUNCPRINT("Don't call me!");
	/*return download_threads_get_info_with_index(index);*/
	return NULL;
}
/*****************************************************
nap_download *transfer_get_info(char *host, char *file)

See above
*****************************************************/
nap_download *transfer_get_info(char *host, char *file)
{
	return download_threads_get_info(host, file);
}
/****************************************************
gboolean transfer_cancel_with_index(int index)

Cancel the specified transfer
****************************************************/
gboolean transfer_cancel_with_index(int index)
{
	FUNCPRINT("Don't call me!");
/*	nap_download *info;
	if((info = download_threads_get_info_with_index(index))==NULL)
	{
		return FALSE;
	}	
	if(download_threads_dequeue_with_index(index))
	{
*/		/* check to see if it is done or still downloading; because if
			it is then we don't need to dec the count */
/*		if(info->status != EG_DOWNLOADING && 
			info->status != EG_FILECOMPLETE)
				num_downloads--;

		g_free(info);
		return TRUE;
	}
	g_free(info);
*/	return FALSE;
}
/***************************************************
gboolean transfer_cancel(char *host, char *file)

See above
***************************************************/
gboolean transfer_cancel(char *host, char *file)
{
	if(download_threads_dequeue(host, file))
	{
		num_downloads--;
		return TRUE;
	}
	return FALSE;
}
void download_done(void)
{
	num_downloads--;
}
void transfer_reset_local_port(int port)
{
	if(localtag>0)
	{	
		FUNCPRINT("Removing old input handler");
		gdk_input_remove(localtag);
	}
	if(nap_socklocal>0)
	{
		FUNCPRINT("Closing old socket");
		close(nap_socklocal);
	}

	FUNCPRINT("Making socket");
	if(nap_bindlocal(port)<0)
	{	
		FUNCPRINT("Error binding to local port");
		return;
	}
	/* setup the input handler on the socket */
	localtag = gdk_input_add(nap_socklocal, 
			GDK_INPUT_READ, transfer_input_accept_cb, 
			NULL);
}

void tr_enum_queue_cb(nap_download *info)
{
	if(info->status == EG_QUEUED && 
		(num_downloads < max_downloads))
	{
		/* we can start it up */
		transfer_send_getinfo_request(info->host,
						info->filename);
		download_threads_change_status(info->host,
				info->filename, EG_GETINFO);
		g_free(info);	
	}
	else if(num_downloads >= max_downloads)
	{
		/* we're already at the limit */
		g_free(info); 
	}
	else g_free(info);
}

gboolean transfer_check_queues(gpointer data)
{
	download_threads_enum_queue(tr_enum_queue_cb);
	return TRUE;
}
void transfer_send_getinfo_request(char *host, char *filename)
{
		FUNCPRINT("Curr: %d\nMax: %d",
			num_downloads, max_downloads);
		FUNCPRINT("Yes, Done");
		/* yes, so send out the request */
		nap_sendpacketf(nap_socknap, NM_FILEINFO_REQUEST, 
				"%s \"%s\"", host, filename);
		num_downloads++;
}
/******************************************************
extern void transfer_add_shared_file(char *filename)

Add a shared file to the list of shared files

RETURNS:
	FALSE if it wasn't able to add the file
	TRUE if it was
******************************************************/
extern gboolean transfer_add_shared_file(char *filename)
{
	return upload_threads_add_shared_file(filename);	
}
