#include "upload_thread.h"
#include "lnap.h"
#include "util.h"
#include "mp3_info.h"
#include "md5sum-mp3.h"
#include <glib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
typedef struct
{
	upload_info file_info;
	/*gint tag;*/
	GIOChannel *channel;
	int sock;
	int filed;
} transfer_info;

GList *shares;
GList *transfers;

transfer_info *ut_transfer_add(char *host, char *filename);
gboolean ut_upload_stop(transfer_info *info); 
gboolean ut_transfer_remove(transfer_info *info);
transfer_info *ut_find_transfer(char *host, char *filename);
share_info *ut_find_share(char *filename);
void ut_uploadproc(transfer_info *d);
void ut_upload_complete(transfer_info *info);
/*static void ut_send_data_cb(gpointer data, gint source, GdkInputCondition
condition);*/
static gboolean ut_send_data_cb(GIOChannel *source, GIOCondition condition,
gpointer data);

/************************************************
************************************************/
gboolean upload_threads_init(void)
{
	FUNCPRINT("Initializing uploading");
	shares = NULL;
	transfers = NULL;
}
/************************************************
************************************************/
gboolean upload_threads_destroy(void)
{
	share_info *info;
	FUNCPRINT("Stopping all threads");
	while(transfers != NULL)
	{
		ut_upload_stop(transfers->data);
		transfers = g_list_next(transfers);
	}
	transfers = NULL;
	
	FUNCPRINT("Removing list of shares");
	while(shares != NULL)
	{
		info = shares->data;
		upload_threads_remove_shared_file(info->share_filename);
		/*shares = g_list_next(shares);*/
	}
	shares = NULL;
	FUNCPRINT("Done");
	return TRUE;
}

/************************************************
************************************************/
GList *upload_threads_get_uploading_list(void)
{
	GList *new = NULL;
	GList *l = transfers;
	upload_info *info;
	transfer_info *data;
	/* make a copy of the list */
	while(l != NULL)
	{
		info = g_malloc0(sizeof(upload_info));
		data = l->data;
		g_assert(data != NULL);
		memcpy(info, &data->file_info, sizeof(upload_info));
		new = g_list_append(new, info);
	
		l = g_list_next(l);
	}
	/* return the copy */
	return new;
}
/************************************************
************************************************/
gboolean upload_threads_stop_upload(char *host, char *filename)
{
	transfer_info *info = ut_find_transfer(host, filename);
	FUNCPRINT("Stoping an upload");
	if(info == NULL)
	{
		FUNCPRINT("That isn't a transfer");
		return FALSE;
	}
	return ut_transfer_remove(info);
}
/************************************************
************************************************/
gboolean upload_threads_start_upload(int sock, char *host, char *filename, long offset)
{
	transfer_info *info;
	FUNCPRINT("Starting a new upload");
	if((info = ut_transfer_add(host, filename)) == NULL)
	{
		FUNCPRINT("Unable to add upload");
		return FALSE;
	}

	info->sock = sock;
	info->file_info.sizecomplete = offset;
	ut_uploadproc(info);
	return TRUE;	
}

/************************************************
************************************************/
gboolean upload_threads_add_shared_file(char *filename)
{
	#define HASH_LENGTH 299008 
	int filed;
	char buffer[MAX_PATH_LENGTH];
	share_info *new=NULL;
	int index1, index2;
	mp3_info *info;
	struct stat stats;
	/* try to open it first */
	if((filed = open(filename, O_RDONLY)) == -1)
	{
		FUNCPRINT("Unable to open the file");
		return FALSE;
	}
	/* add the node to the list */
	new = g_malloc0(sizeof(share_info));

	/*fill in some info*/
	strncpy(new->actual_filename, filename, MAX_PATH_LENGTH);
	
	
/*NAME MANGLING: Change the name to look like a windows name
	so add "C:" to the front and change all "/" to "\".  
	Remove all illegal characters as well. */
	new->share_filename[0] = 'C';
	new->share_filename[1] = ':';
	for(index1 = 2, index2 = 0;index1<MAX_PATH_LENGTH;index1++, index2++)
	{
		/*FIXME: what else do I need to check for? */
		if(filename[index2] == '/')
			new->share_filename[index1] = '\\';
		else if(filename[index2] == '\0')
			break;
		else
			new->share_filename[index1] = filename[index2];
	}

	/* get the md5 of the file  */
	sum(filed, HASH_LENGTH, new->md5); 
	/* get the other info about it */
	lseek(filed, 0L, SEEK_SET);
	if((info = mp3_get_info(filed)) == NULL)
	{
		FUNCPRINT("Unable to get info about the mp3");
		g_free(new);
		return FALSE;
	}
	new->bitrate = info->bitrate;
	new->frequency = info->frequency;
	new->length = info->length;
	new->filesize = info->file_length;	

	shares = g_list_append(shares, new);
	g_free(info);

	/* send the message to the server */
	sprintf(buffer, "\"%s\" %s %d %d %d %d", new->share_filename,
						new->md5,
						new->filesize,	
						new->bitrate,
						new->frequency,
						new->length);
	nap_sendpacket(nap_socknap, NM_SHARED_FILE_NOTIFY, strlen(buffer),
								buffer);
	FUNCPRINT("Sent: %s", buffer);
	return TRUE;
}
/************************************************
************************************************/
gboolean upload_threads_remove_shared_file(char *filename)
{
	/* find the node */
	share_info *share = ut_find_share(filename);
	if(share == NULL)
	{
		FUNCPRINT("File not shared");
		return FALSE;
	}
	/* send the message to the server */
	nap_sendpacketf(nap_socknap, NM_SHARED_FILE_REMOVE, "%s",
						share->share_filename); 
	/* remove the node */
	shares = g_list_remove(shares, share);
	g_free(share);
	return TRUE;
}
/************************************************
************************************************/
GList *upload_threads_get_share_list(void)
{
	/* make a copy of the list and return it */
	GList *new = NULL;
	GList *l = shares;
	share_info *info;
	/* make a copy of the list */
	while(l != NULL)
	{
		info = g_malloc0(sizeof(share_info));
		memcpy(info, l->data, sizeof(share_info));
		new = g_list_append(new, info);
	
		l = g_list_next(l);
	}
	/* return the copy */
	return new;

}
/************************************************
************************************************/
share_info *upload_threads_get_share_info(char *filename)
{
	/* find the node */
	share_info *info = ut_find_share(filename);
	share_info *copy;
	if(info == NULL)
	{
		FUNCPRINT("Unable to find share");
		return NULL;
	}
	FUNCPRINT("Making copy");
	/* make a copy of the info */
	copy = g_malloc0(sizeof(share_info));
	memcpy(copy, info, sizeof(share_info));
	/* return it */
	return copy;
}

/************************************************
************************************************/
transfer_info *ut_transfer_add(char *host, char *filename)
{
	transfer_info *info = NULL;
	share_info *share = NULL;
	/* allocate the array */
	FUNCPRINT("Adding %s, %s", host, filename);
	info = g_malloc0(sizeof(transfer_info));
	share = upload_threads_get_share_info(filename);
	/* copy over the info */
	strncpy(info->file_info.host, host, MAX_HOST_LENGTH);
	strncpy(info->file_info.filename, share->share_filename, MAX_PATH_LENGTH);
	strncpy(info->file_info.realname, share->actual_filename,
MAX_PATH_LENGTH);
	g_free(share);
	/* add it onto the list */
	transfers = g_list_append(transfers, info);
	return info;
}
/************************************************
************************************************/
gboolean ut_upload_stop(transfer_info *info)
{
	GList *node;
	/*if(info->tag)
		ut_upload_complete(info);*/
/*	else
	{
		transfers = g_list_remove(transfers, info);
		g_free(info);
	}*/
	return TRUE;
}
/************************************************
************************************************/
gboolean ut_transfer_remove(transfer_info *info)
{
	FUNCPRINT("Removing transfer");
	transfers = g_list_remove(transfers, info);
	g_free(info);
	return TRUE;	
}
/************************************************
************************************************/
transfer_info *ut_find_transfer(char *host, char *filename)
{
	GList *l = transfers;
	transfer_info *data;
	while(l != NULL)
	{
		data = l->data;
		if(!strcmp(host, data->file_info.host) &&
			!strcmp(filename, data->file_info.filename))
		{
			FUNCPRINT("Found it");
			return data;
		}	
		l = g_list_next(l);
	}
	return NULL;
}
/************************************************
************************************************/
void ut_uploadproc(transfer_info *info)
{
	/* open the file */
	FUNCPRINT("Trying to open %s", info->file_info.realname);
	if((info->filed = open(info->file_info.realname, O_RDONLY))<0)
	{
		FUNCPRINT("Unable to open the file for the transfer");
		close(info->sock);
		return;
	} 
	/* seek to the offset */
	if(lseek(info->filed, info->file_info.sizecomplete, SEEK_SET) < 0)
	{
		FUNCPRINT("Unable to seek to the correct spot on the file");
		close(info->filed);
		close(info->sock);
		return;
	}
	/* send the info */
	FUNCPRINT("Sending file");
	info->channel = g_io_channel_unix_new(info->sock);
	g_io_add_watch_full(info->channel, 1, G_IO_OUT | G_IO_HUP, ut_send_data_cb,
info, ut_upload_complete);
	/*info->tag = gdk_input_add(info->sock, GDK_INPUT_WRITE, ut_send_data_cb,
									info);*/
	
	FUNCPRINT("Input handler setup");
}
static gboolean ut_send_data_cb(GIOChannel *source, GIOCondition condition,
gpointer data)
{
	#define BUF_SIZE 1024
	transfer_info *info = data;
	gchar buffer[BUF_SIZE];
	guint len;
	guint written;
	if(condition == G_IO_HUP)
		return FALSE;

/*	FUNCPRINT("Reading info");*/
	if((len = read(info->filed, buffer, BUF_SIZE))>0)
	{
/*		FUNCPRINT("Sending %d bytes", len);*/
		/* TODO: need to add throttling */
		/* TODO: do kbps calc */
		/*if(send(info->sock, buffer, len, 0)<0)*/
		if(g_io_channel_write(info->channel, buffer, len, &written) !=
							G_IO_ERROR_NONE) 
		{
			FUNCPRINT("Error on send, stopping");
			/*ut_upload_complete(info);*/
			return FALSE;
		}
	}
	else
	{
		FUNCPRINT("Done sending");
		return FALSE;
	}
	return TRUE;
}
void ut_upload_complete(transfer_info *info)
{
	FUNCPRINT("Cleaning up upload");
	/*gdk_input_remove(info->tag);*/
	g_io_channel_close(info->channel);
	g_io_channel_unref(info->channel);
	if(info->filed)
		close(info->filed);
	info->filed = 0;
	if(info->sock)
		close(info->sock);
	info->sock = 0;
	FUNCPRINT("Done");
}
/*****************************************
*****************************************/
share_info *ut_find_share(char *filename)
{
	GList *l = shares;
	share_info *info;
	FUNCPRINT("Searching for %s in shares", filename);
	while(l != NULL)
	{
		info = l->data;
		if(!strcmp(info->actual_filename, filename) ||
			!strcmp(info->share_filename, filename))
		{		
			FUNCPRINT("Found it");
			return info;
		}
		g_assert(info != NULL);
		
		l = g_list_next(l);
	}
	FUNCPRINT("Wasn't here");
	return NULL;
}
