#include "gtkflist.h"
#include "util.h"
#include <gnome.h>
#include <glib.h>
#include "transfer.h"
#include "upload_thread.h"
#include "upload_ui.h"
static char* titles[] = { "File", "Host", "Status", 
				"Bitrate", "MD5", "Frequency" };

enum {
	COL_TITLE, COL_HOST, COL_STATUS, 
	COL_BITRATE, COL_MD5, COL_FREQUENCY, 
	COL_COUNT
};

typedef struct
{
	char host[MAX_HOST_LENGTH];
	char filename[MAX_PATH_LENGTH];
} upload_entry;

GtkWidget *fileselector;
gchar *fileselection;

static GtkWidget *upload_clist, *share_clist;

static GtkWidget *upload_transfer_list_create(void);
static GtkWidget *upload_share_list_create(void);
static void up_share_ui_update(void);
static gboolean upload_ui_update(gpointer data);
static void up_transfer_ui_update(void);
static void upload_ui_cancel_cb(GtkWidget *w, gpointer d);



/* popup text */
static GnomeUIInfo popup_menu[] = {
/*
	GNOMEUIINFO_ITEM_NONE(N_("_Select All"),
		N_("Select all items in the list"),
		selall_cb),

	GNOMEUIINFO_ITEM_NONE(N_("_Song Info"),
		N_("View information about this download"),
		song_info_cb),
*/
	GNOMEUIINFO_ITEM_NONE(N_("_Cancel"),
		N_("Cancel this download"),
		upload_ui_cancel_cb),
/*	GNOMEUIINFO_ITEM_NONE(N_("_Host Info"),
		N_("View information about this host"),
		host_info_cb),
	GNOMEUIINFO_END*/
};

/********************************************************************
gboolean upload_ui_download_add(char *host, char *title, int shift)

Add a new download to the clist and start the transfer.
*********************************************************************/
/*gboolean upload_ui_download_add(char *host, char *title, int shift)
{
	download_entry *entry = NULL;
	int row;
	gchar *row_text[COL_COUNT] = {"", "", ""};

	entry = g_malloc0(sizeof(download_entry));
	strncpy(entry->host, host, MAX_HOST_LENGTH);
	strncpy(entry->filename, title, MAX_PATH_LENGTH);
	
	if((entry->index = transfer_get_file(host, title, shift))<0)
	{
		FUNCPRINT("Error adding file");
		g_free(entry);
		return FALSE;
	}
	row_text[COL_TITLE] = short_path(entry->filename, shift);
	row_text[COL_HOST]  = entry->host;

	row = gtk_clist_append(GTK_CLIST(clist), row_text);
	gtk_clist_set_row_data(GTK_CLIST(clist), row, entry);
	
	return TRUE;
}*/
static void upload_store_filename_cb(GtkFileSelection *w, gpointer d)
{
	fileselection =
	gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileselector));
	FUNCPRINT("Selection is %s", fileselection);
	if(!upload_threads_add_shared_file(fileselection))
	{
		FUNCPRINT("Something went wrong adding it");
		return;
	}
	FUNCPRINT("Added");
}
/***********************************************************
static void upload_add_file_cb(GtkWidget *w, gpointer d)
***********************************************************/
static void upload_add_file_cb(GtkWidget *w, gpointer d)
{
	fileselector = gtk_file_selection_new("Add shared file");
	
	gtk_signal_connect_object(
		GTK_OBJECT(GTK_FILE_SELECTION(fileselector)->ok_button),
		"clicked", GTK_SIGNAL_FUNC(upload_store_filename_cb), NULL);
	
	gtk_signal_connect_object(
		GTK_OBJECT(GTK_FILE_SELECTION(fileselector)->ok_button),
		"clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
		(gpointer)fileselector);
	gtk_signal_connect_object(
		GTK_OBJECT(GTK_FILE_SELECTION(fileselector)->cancel_button),
		"clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
		(gpointer)fileselector);
	gtk_widget_show(fileselector);
}
/***********************************************************
static void upload_cancel_cb(GtkWidget *w, gpointer d)

Callback for canceling downloads
***********************************************************/
static void upload_ui_cancel_cb(GtkWidget *w, gpointer d)
{
	int row;
	GList *sel;
	upload_entry *entry;

	FUNCPRINT("Canceling Upload");
	/* we'll be using the clist */
 	sel = GTK_CLIST(upload_clist)->selection;
	if(sel == NULL) 
	{
		FUNCPRINT("Nothing selected");
		return;	
	}
	entry = gtk_clist_get_row_data(GTK_CLIST(upload_clist), (int)sel->data); 
	g_assert(entry != NULL);


	/* send the cancel request to the thread */
	transfer_cancel(entry->host, entry->filename);
	/* remove the entry from the clist */
	gtk_clist_remove(GTK_CLIST(upload_clist), (int)sel->data); 
	g_free(entry);
	FUNCPRINT("Done");	
}
/*************************************************************
static void upload_remove_complete_cb(GtkWidget *w, gpointer d)

Callback to remove all entries in the clist that are marked as "File Complete".
*************************************************************/
static void upload_ui_remove_complete_cb(GtkWidget *w, gpointer d)
{
	int row = 0;
	upload_entry *entry;
	nap_download *info;
	FUNCPRINT("Canceling Finished Uploads");
	/* we'll be using the clist */
	entry=gtk_clist_get_row_data(GTK_CLIST(upload_clist), row); 	
	do{
		if((info = transfer_get_info(entry->host, entry->filename))==NULL)
		{
			gtk_clist_remove(GTK_CLIST(upload_clist), row);
			break;
		}
		if(info->status == EG_FILECOMPLETE)
		{
			FUNCPRINT("Removing completed upload");
			transfer_cancel(info->host, info->filename);
			gtk_clist_remove(GTK_CLIST(upload_clist), row);
			g_free(info);
			g_free(entry);
		}
 		else
			row++;

		entry=gtk_clist_get_row_data(GTK_CLIST(upload_clist), row); 
	} while(entry != NULL);

	FUNCPRINT("Done");	
	
}
/***********************************************************
static void upload_ui_button_press_cb(GtkWidget *w, GdkEventButton *e, gpointer d)

Show the popup menu
***********************************************************/
static gint upload_ui_button_press_cb(GtkWidget *w, GdkEventButton *e, gpointer d) {
	/*FIXME: don't do anything at the moment */
}
/***********************************************************
GtkWidget* uploadtab_create(void)

Make the upload tab
***********************************************************/
GtkWidget* uploadtab_create(void) {
	GtkWidget *vbox;
	vbox = gtk_vbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), upload_transfer_list_create(), 
					TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), upload_share_list_create(),
					TRUE, TRUE, 0);
	g_timeout_add(1000, upload_ui_update, NULL);	
	return vbox;
}
/*************************************************
GtkWidget *upload_transfer_list_create()
**************************************************/
static GtkWidget *upload_transfer_list_create(void)
{
	GtkWidget *scroll;
	GtkWidget *vbox, *hbox;
	GtkWidget *b;

	vbox = gtk_vbox_new(FALSE, 10);
/* the buttons on top */
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
		hbox = gtk_hbox_new(FALSE, 10);
		/* Make the button for canceling downloads */
			b = gtk_button_new_with_label("Remove");
		gtk_box_pack_start(GTK_BOX(hbox), b, FALSE, FALSE, 0);
		gtk_signal_connect(GTK_OBJECT(b), "clicked", 
				GTK_SIGNAL_FUNC(upload_ui_cancel_cb), NULL);
		/* Make the button to cancel completed downloads */
			b = gtk_button_new_with_label("Remove Complete");
		gtk_box_pack_start(GTK_BOX(hbox), b, FALSE, FALSE, 0);
		gtk_signal_connect(GTK_OBJECT(b), "clicked",
			GTK_SIGNAL_FUNC(upload_ui_remove_complete_cb), NULL);
		/* let it do things */
		gtk_widget_set_sensitive(hbox, TRUE);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

/* the clist on bottom */
		scroll = gtk_scrolled_window_new(NULL, NULL);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll), 
				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
			upload_clist = gtk_flist_new_with_titles(sizeof(titles)/sizeof(char*), titles);
			gtk_clist_set_column_width(GTK_CLIST(upload_clist), COL_TITLE, 300);
			gtk_clist_set_column_width(GTK_CLIST(upload_clist), COL_HOST, 50);

			gtk_signal_connect(GTK_OBJECT(upload_clist), "button_press_event",
				GTK_SIGNAL_FUNC(upload_ui_button_press_cb), NULL);

		gtk_container_add(GTK_CONTAINER(scroll), upload_clist);
	gtk_box_pack_start(GTK_BOX(vbox), scroll, TRUE, TRUE, 0);

	return vbox;
}
/*************************************************
GtkWidget *upload_share_list_create()
*************************************************/
static GtkWidget *upload_share_list_create(void)
{
	GtkWidget *scroll;
	GtkWidget *vbox, *hbox;
	GtkWidget *b;

	vbox = gtk_vbox_new(FALSE, 10);
/* the buttons on top */
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
		hbox = gtk_hbox_new(FALSE, 10);
		/* Make the button to add a share */
			b = gtk_button_new_with_label("Add share");
		gtk_box_pack_start(GTK_BOX(hbox), b, FALSE, FALSE, 0);
		gtk_signal_connect(GTK_OBJECT(b), "clicked", 
			GTK_SIGNAL_FUNC(upload_add_file_cb), NULL);
		/* let it do things */
		gtk_widget_set_sensitive(hbox, TRUE);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

/* the clist on bottom */
		scroll = gtk_scrolled_window_new(NULL, NULL);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll), 
				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
			share_clist = gtk_flist_new_with_titles(sizeof(titles)/sizeof(char*), titles);
			gtk_clist_set_column_width(GTK_CLIST(share_clist), COL_TITLE, 300);
			gtk_clist_set_column_width(GTK_CLIST(share_clist), COL_HOST, 50);

			gtk_signal_connect(GTK_OBJECT(share_clist), "button_press_event",
				GTK_SIGNAL_FUNC(upload_ui_button_press_cb), NULL);

		gtk_container_add(GTK_CONTAINER(scroll), share_clist);
	gtk_box_pack_start(GTK_BOX(vbox), scroll, TRUE, TRUE, 0);

	return vbox;
}
/*************************************************
gboolean upload_ui_update(gpointer data)

The callback to update the upload screen
*************************************************/
static gboolean upload_ui_update(gpointer data)
{
	up_share_ui_update();
	up_transfer_ui_update(); 
	return TRUE;	
}
/***********************************************
static void up_share_ui_update(void)
***********************************************/
static void up_share_ui_update(void)
{
	share_info *info = NULL;
	GList *share_list = NULL;
	GList *l = NULL;
	gchar **text;
	gchar *strings[] = {NULL, NULL, NULL, NULL, NULL, NULL};
	int index;
	gtk_clist_freeze(GTK_CLIST(share_clist));
	/* clear out the list */
	for(index = 0; gtk_clist_get_text(GTK_CLIST(share_clist), 
						index, 0, text);)
	{
		gtk_clist_remove(GTK_CLIST(share_clist), index);	
	}
	share_list = upload_threads_get_share_list();
	l = share_list;
	while(l != NULL)
	{
		info = (share_info *)l->data;
		strings[COL_TITLE] = info->actual_filename;
		strings[COL_BITRATE] = g_strdup_printf("%d", info->bitrate);
		strings[COL_FREQUENCY] = g_strdup_printf("%d",
info->frequency);
		strings[COL_MD5] = info->md5;
		gtk_clist_append(GTK_CLIST(share_clist), strings);
		g_free(strings[COL_BITRATE]);
		g_free(strings[COL_FREQUENCY]);
		g_free(l->data);
		l = g_list_next(l);
	}
	g_list_free(share_list);
	gtk_clist_thaw(GTK_CLIST(share_clist));
}
/***********************************************
static void up_transfer_ui_update(void)
***********************************************/
static void up_transfer_ui_update(void)
{
	upload_info *info = NULL;
	GList *upload_list = NULL;
	GList *l = NULL;
	gchar **text;
	gchar *strings[] = {NULL, NULL, NULL, NULL, NULL, NULL};
	int index;
	gtk_clist_freeze(GTK_CLIST(upload_clist));
	/* clear out the list */
	for(index = 0; gtk_clist_get_text(GTK_CLIST(upload_clist), 
						index, 0, text);)
	{
		gtk_clist_remove(GTK_CLIST(upload_clist), index);	
	}
	upload_list = upload_threads_get_uploading_list();
	l = upload_list;
	while(l != NULL)
	{
		info = (upload_info *)l->data;
		strings[COL_HOST] = info->host;
		strings[COL_TITLE] = info->realname;
		FUNCPRINT("%s@%s", strings[COL_TITLE], strings[COL_HOST]);
	/*	strings[COL_STATUS] = g_strdup_printf("%d", info->kbps);*/	
		gtk_clist_append(GTK_CLIST(upload_clist), strings);
	/*	g_free(strings[COL_STATUS]); */
		g_free(info);
		l = g_list_next(l);
	}
	g_list_free(upload_list);
	gtk_clist_thaw(GTK_CLIST(upload_clist));
}
