/* gnome-napster, a GNOME Napster client.
 * Copyright (C) 1999 Evan Martin <eeyem@u.washington.edu>
 * Copyright (C) 2000 Andrew Parker <zaphod42@u.washington.edu>
 */
#include <gnome.h>
#include <unistd.h>
#include <fcntl.h>
#include "util.h"
#include "gconfig.h"

int g_quiet = 0;

int make_blocking(int sock)
{
	int flags;
	/* read in the current options */
	if((flags = fcntl(sock, F_GETFL)) < 0)
	{
		FUNCPRINT("Couldn't get socket options!");	
		return -1;
	}
	if(flags & O_NONBLOCK)
	{
		FUNCPRINT("it's NONBLOCKING")
		/* unset the nonblocking bit */
		if(fcntl(sock, F_SETFL, (flags ^ O_NONBLOCK)) < 0)
		{
			FUNCPRINT("Couldn't set socket options");
			return -1;
		}
	}
	return sock;
}
int make_nonblocking(int sock)
{
	int flags;
	/* read in the current options */
	if((flags = fcntl(sock, F_GETFL)) < 0)
	{
		FUNCPRINT("Couldn't get socket options!");	
		return -1;
	}
	/* set the nonblocking bit */
	if(fcntl(sock, F_SETFL, flags | O_NONBLOCK) < 0)
	{
		FUNCPRINT("Couldn't set socket options");
		return -1;
	}
	return sock;
}

void qprintf(char *fmt, ...) {
	va_list ap;
	if (!g_quiet) {
		va_start(ap, fmt);
		vprintf(fmt, ap);
		va_end(ap);
	}
}

char *short_path(char *path, int depth) {
	char *p = path+strlen(path)-1;
	for ( ; p >= path; p--) {
		if ((*p == '\\') || (*p == '/'))  {
			depth--;
			if (depth < 0) break;
		}
	}
	if (p > path) 
		p++; /* skip the leftmost path separator */
	if (p == 0 || p < path) p = path; /* watch out for weird condititions */
	return p;
}

GtkWidget* hbox_new_with_label(gchar *text, GtkWidget *widg, gboolean exp) {
	GtkWidget *hbox;
	hbox = gtk_hbox_new(FALSE, 10);
	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(text), FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), widg, exp, exp, 0);
	return hbox;
}

gchar *pixmap_file(char *fn) {
	char *file;
	if ((file = gnome_pixmap_file(fn)) == NULL) {
		/* if it's not in the pixmap path, try a relative path.
		 * it works for the animator, at least! */
		file = g_strconcat("../pixmaps/", fn, NULL);
	} 
	return file;
}

GList* g_list_free_all(GList *list) {
	GList *n;

	while (list != NULL) {
		g_free(list->data);
		n = g_list_next(list);
		g_list_free_1(list);
		list = n;
	}
	return NULL; /* the list is gone... kind of a useless response, but... */
}

/**************************************************
gboolean behind_firewall(void)

Return TRUE if the behind firewall checkbox is checked,
FALSE otherwise.
**************************************************/
gboolean behind_firewall(void)
{	
	return gnome_config_get_int(CONFIG_PREFIX "/Login/Firewall=0");
}
